@@ -1,4 +1,4 @@
-/* Copyright IBM Corp. 2021
+/* Copyright IBM Corp. 2021, 2024
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -39,6 +39,7 @@
 #include "lib/util_scandir.h"
 #include "lib/util_libc.h"
 #include "lib/util_file.h"
+#include "lib/util_fmt.h"
 #include "lib/libcpumf.h"
 
 #include "lshwc.h"
@@ -51,18 +52,91 @@
 
 static unsigned int read_interval = IOCTLSLEEP;
 static int cfvn, csvn, authorization;
-static unsigned long loop_count = 1;
+static unsigned long loop_count = 1, timeout;
 static unsigned char *ioctlbuffer;
 static bool allcpu;
+static char *ctrformat = "%ld";
+static bool shortname;
+static bool hideundef;
+static bool delta, firstread;
+static int output_format = FMT_CSV;
+static bool quote_all;
 
 static unsigned int max_possible_cpus;	/* No of possible CPUs */
 static struct ctrname {		/* List of defined counters */
 	char *name;		/* Counter name */
+	char *label;		/* Output name */
 	bool hitcnt;		/* Counter number read from ioctl() */
 	unsigned long total;	/* Total counter value */
 	unsigned long *ccv;	/* Per CPU counter value */
+	unsigned long *ccvprv;	/* Per CPU counter value (previous read) */
 } ctrname[MAXCTRS];
 
+struct time_formats {
+	char epoch[32];
+	char date_time[32];
+	char date[16];
+	char time[16];
+};
+
+static void mk_labels(void)
+{
+	char label[64];
+	size_t i;
+
+	for (i = 0; i < ARRAY_SIZE(ctrname); ++i) {
+		if (shortname) {
+			if (ctrname[i].name)
+				snprintf(label, sizeof(label), "%s", ctrname[i].name);
+			else
+				snprintf(label, sizeof(label), "U%ld", i);
+		} else {
+			if (output_format == FMT_CSV)
+				snprintf(label, sizeof(label), "%s(%ld)",
+					 ctrname[i].name ?: "Counter", i);
+			else if (ctrname[i].name)
+				snprintf(label, sizeof(label), "%s", ctrname[i].name);
+			else
+				label[0] = 0;
+		}
+		if (output_format != FMT_CSV)
+			util_str_tolower(label);
+		ctrname[i].label = util_strdup(label);
+	}
+}
+
+static char *mk_name(int ctr, char *name)
+{
+	char ctrset[8];
+
+	if (!shortname)
+		return util_strdup(name);
+
+	switch (libcpumf_ctrset(ctr, cfvn, csvn)) {
+	case CPUMF_CTRSET_BASIC:
+		ctrset[0] = 'B';
+		break;
+	case CPUMF_CTRSET_PROBLEM_STATE:
+		ctrset[0] = 'P';
+		break;
+	case CPUMF_CTRSET_CRYPTO:
+		ctrset[0] = 'C';
+		break;
+	case CPUMF_CTRSET_EXTENDED:
+		ctrset[0] = 'E';
+		break;
+	case CPUMF_CTRSET_MT_DIAG:
+		ctrset[0] = 'M';
+		break;
+	default:
+		ctrset[0] = 'U';
+		break;
+	}
+	sprintf(ctrset, "%c%d", ctrset[0], ctr);
+
+	return util_strdup(ctrset);
+}
+
 static bool read_counternames(void)
 {
 	struct dirent **namelist = NULL;
@@ -79,7 +153,7 @@ static bool read_counternames(void)
 	for (i = 0; i < count && ctr >= 0; i++) {
 		util_asprintf(&ctrpath, "%s/%s", path, namelist[i]->d_name);
 		if (util_file_read_va(ctrpath, "event=%x", &ctr) == 1)
-			ctrname[ctr].name = util_strdup(namelist[i]->d_name);
+			ctrname[ctr].name = mk_name(ctr, namelist[i]->d_name);
 		else
 			warnx("Cannot parse %s", ctrpath);
 		free(ctrpath);
@@ -93,7 +167,9 @@ static void free_counternames(void)
 {
 	for (size_t i = 0; i < ARRAY_SIZE(ctrname); ++i) {
 		free(ctrname[i].name);
+		free(ctrname[i].label);
 		free(ctrname[i].ccv);
+		free(ctrname[i].ccvprv);
 	}
 }
 
@@ -298,77 +374,111 @@ static bool check_setpossible(void)
 	return true;
 }
 
-static void show_header(void)
+static void safe_strtime(char *dest, size_t size, const char *fmt, const struct tm *tm)
 {
-	static bool header;
-	bool comma = false;
-
-	if (header)
-		return;			/* Printed already */
-	printf("Date,Time,CPU,");	/* Print counter name and number */
-	for (size_t i = 0; i < ARRAY_SIZE(ctrname); ++i) {
-		if (!ctrname[i].hitcnt)
-			continue;
-		if (comma)
-			putchar(',');
-		printf("%s(%ld)", ctrname[i].name ?: "Counter", i);
-		comma = true;
-	}
-	putchar('\n');
-	header = true;
+	if (!strftime(dest, size, fmt, tm))
+		dest[0] = 0;
 }
 
-static void line(char *header)
+static void generate_timestamp(struct time_formats *date)
 {
-	bool comma;
+	time_t now = time(NULL);
+	struct tm *now_tm = localtime(&now);
 
-	show_header();
-	if (allcpu) {
-		for (unsigned int h = 0; h < max_possible_cpus; ++h) {
-			char txt[16];
+	safe_strtime(date->date_time, sizeof(date->date_time), "%F %T%z", now_tm);
+	safe_strtime(date->date, sizeof(date->date), "%F", now_tm);
+	safe_strtime(date->time, sizeof(date->time), "%T", now_tm);
+	safe_strtime(date->epoch, sizeof(date->epoch), "%s", now_tm);
+}
 
-			if (!check[h].cpu_hit)
-				continue;
-			comma = false;
-			snprintf(txt, sizeof(txt), "CPU%d,", h);
-			printf("%s%s", header, txt);
-			for (size_t i = 0; i < ARRAY_SIZE(ctrname); ++i) {
-				if (!ctrname[i].hitcnt)
-					continue;
-				if (comma)
-					putchar(',');
-				printf("%ld", ctrname[i].ccv[h]);
-				comma = true;
-			}
-			putchar('\n');
-		}
+static void output_times(struct time_formats date)
+{
+	if (output_format == FMT_CSV) {
+		util_fmt_pair(FMT_PERSIST, "Date", "%s", date.date);
+		util_fmt_pair(FMT_PERSIST, "Time", "%s", date.time);
+	} else {
+		util_fmt_pair(FMT_PERSIST | FMT_QUOTE, "date_time", "%s", date.date_time);
+		util_fmt_pair(FMT_PERSIST, "time_epoch", "%s", date.epoch);
 	}
+}
 
-	/* Print total count of all CPUs */
-	printf("%sTotal,", header);
-	comma = false;
+static void prepare_counter(size_t id, unsigned long value)
+{
+	if (output_format == FMT_CSV) {
+		util_fmt_pair(FMT_PERSIST, ctrname[id].label, ctrformat, value);
+	} else {
+		util_fmt_obj_start(FMT_ROW, NULL);
+		if (strlen(ctrname[id].label))
+			util_fmt_pair(FMT_PERSIST | FMT_QUOTE, "name", ctrname[id].label);
+		util_fmt_pair(FMT_PERSIST, "id", ctrformat, id);
+		util_fmt_pair(FMT_PERSIST, "value", ctrformat, value);
+		util_fmt_obj_end();
+	}
+}
+
+static void output_per_cpu(struct time_formats date)
+{
+	for (unsigned int h = 0; h < max_possible_cpus; ++h) {
+		if (!check[h].cpu_hit)
+			continue;
+
+		char txt[16];
+
+		snprintf(txt, sizeof(txt), "CPU%d", h);
+		util_fmt_obj_start(FMT_ROW, "cpu_%d", h);
+		output_times(date);
+		if (output_format == FMT_CSV) {
+			util_fmt_pair(FMT_PERSIST, "CPU", "CPU%d", h);
+		} else {
+			util_fmt_pair(FMT_PERSIST, "cpu", "%d", h);
+			util_fmt_obj_start(FMT_LIST, "counters");
+		}
+		for (size_t i = 0; i < ARRAY_SIZE(ctrname); ++i) {
+			if (!ctrname[i].hitcnt)
+				continue;
+			if (hideundef && !ctrname[i].name)
+				continue;
+			prepare_counter(i, ctrname[i].ccv[h]);
+		}
+		if (output_format != FMT_CSV)
+			util_fmt_obj_end();
+		util_fmt_obj_end();
+	}
+}
+
+static void output_total(struct time_formats date)
+{
+	util_fmt_obj_start(FMT_ROW, "total");
+	output_times(date);
+	if (output_format == FMT_CSV) {
+		util_fmt_pair(FMT_PERSIST, "CPU", "%s", delta && !firstread ? "Delta" : "Total");
+	} else {
+		util_fmt_pair(FMT_PERSIST | FMT_QUOTE, "cpu", "%s",
+			      delta && !firstread ? "delta" : "total");
+		util_fmt_obj_start(FMT_LIST, "counters");
+	}
 	for (size_t i = 0; i < ARRAY_SIZE(ctrname); ++i) {
 		if (!ctrname[i].hitcnt)
 			continue;
-		if (comma)
-			putchar(',');
-		printf("%ld", ctrname[i].total);
-		comma = true;
+		if (hideundef && !ctrname[i].name)
+			continue;
+		prepare_counter(i, ctrname[i].total);
 		ctrname[i].total = 0;
 		ctrname[i].hitcnt = false;
 	}
-	putchar('\n');
+	if (output_format != FMT_CSV)
+		util_fmt_obj_end();
+	util_fmt_obj_end();
 }
 
-static void show(void)
+static void show_format(void)
 {
-	time_t now = time(NULL);
-	struct tm *now_tm;
-	char now_text[32];
+	struct time_formats now;
 
-	now_tm = localtime(&now);
-	strftime(now_text, sizeof(now_text), "%F,%T,", now_tm);
-	line(now_text);
+	generate_timestamp(&now);
+	if (allcpu)
+		output_per_cpu(now);
+	output_total(now);
 }
 
 /* Return Counter set size numbers (in counters) */
@@ -442,11 +552,18 @@ static bool add_countervalue(size_t idx, unsigned int cpu, unsigned long value)
 		warnx("Invalid CPU number %d", cpu);
 		return false;
 	}
-	if (!ctrname[idx].ccv)		/* Unknown counter */
-		ctrname[idx].ccv = calloc(max_possible_cpus,
-					  sizeof(unsigned long));
-	if (ctrname[idx].ccv)
+	if (delta) {
+		if (firstread) {
+			ctrname[idx].ccvprv[cpu] = value;
+			ctrname[idx].ccv[cpu] = value;
+		} else {
+			ctrname[idx].ccv[cpu] = value - ctrname[idx].ccvprv[cpu];
+			ctrname[idx].ccvprv[cpu] = value;
+			value = ctrname[idx].ccv[cpu];
+		}
+	} else {
 		ctrname[idx].ccv[cpu] = value;
+	}
 	ctrname[idx].total += value;
 	ctrname[idx].hitcnt = true;
 	return true;
@@ -493,7 +610,8 @@ static int test_read(struct s390_hwctr_read *read)
 			}
 		}
 	}
-	show();
+	show_format();
+	firstread = false;
 	return 0;
 }
 
@@ -556,6 +674,7 @@ static void do_sleep(void)
 static int do_it(char *s)
 {
 	struct s390_hwctr_start start;
+	unsigned int flags = FMT_WARN;
 	int ioctlfd;
 	int rc;
 
@@ -575,6 +694,24 @@ static int do_it(char *s)
 		return EXIT_FAILURE;
 	}
 
+	if (output_format == FMT_CSV)
+		flags |= FMT_NOMETA;
+	if (output_format == FMT_JSON || output_format == FMT_JSONSEQ)
+		flags |= FMT_HANDLEINT;
+	if (quote_all)
+		flags |= FMT_QUOTEALL;
+
+	mk_labels();
+	util_fmt_init(stdout, output_format, flags, 1);
+	util_fmt_obj_start(FMT_DEFAULT, "lshwc");
+	if (output_format == FMT_JSON || output_format == FMT_JSONSEQ) {
+		util_fmt_obj_start(FMT_ROW, "cpumcf info");
+		util_fmt_pair(FMT_PERSIST, "counter first", "%d", cfvn);
+		util_fmt_pair(FMT_PERSIST, "counter second", "%d", csvn);
+		util_fmt_pair(FMT_PERSIST, "authorization", "%d", authorization);
+		util_fmt_obj_end();
+	}
+	util_fmt_obj_start(FMT_LIST, "measurements");
 	for (unsigned long i = 0; !rc && i < loop_count; ++i) {
 		rc = do_read(ioctlfd);
 		if (rc) {
@@ -584,6 +721,9 @@ static int do_it(char *s)
 		if (read_interval && i + 1 < loop_count)
 			do_sleep();
 	}
+	util_fmt_obj_end();
+	util_fmt_obj_end();
+	util_fmt_exit();
 	rc = do_stop(ioctlfd);
 	close(ioctlfd);
 	return rc ? EXIT_FAILURE : EXIT_SUCCESS;
@@ -605,6 +745,40 @@ static struct util_opt opt_vec[] = {
 		.argument = "NUMBER",
 		.desc = "Specifies interval between read operations (seconds)"
 	},
+	{
+		.option = { "short", no_argument, NULL, 's' },
+		.desc = "Abbreviate counter name with counter set letter and number"
+	},
+	{
+		.option = { "hex0x", no_argument, NULL, 'X' },
+		.desc = "Counter values in hexadecimal format with leading 0x"
+	},
+	{
+		.option = { "hex", no_argument, NULL, 'x' },
+		.desc = "Counter values in hexadecimal format"
+	},
+	{
+		.option = { "hide", no_argument, NULL, 'H' },
+		.desc = "Do not display undefined counters of a counter set"
+	},
+	{
+		.option = { "delta", no_argument, NULL, 'd' },
+		.desc = "Display delta counter values"
+	},
+	{
+		.option = { "timeout", required_argument, NULL, 't' },
+		.argument = "NUMBER",
+		.desc = "run time in s (seconds) m (minutes) h (hours) and d (days)"
+	},
+	{
+		.option = { "quote-all", no_argument, NULL, 'q' },
+		.desc = "Apply quoting to all output elements"
+	},
+	{
+		.option = { "format", required_argument, NULL, 'f' },
+		.argument = "FORMAT",
+		.desc = "List counters in specified FORMAT (" FMT_TYPE_NAMES ")"
+	},
 	UTIL_OPT_HELP,
 	UTIL_OPT_VERSION,
 	UTIL_OPT_END
@@ -634,6 +808,8 @@ static void have_support(void)
 
 int main(int argc, char **argv)
 {
+	enum util_fmt_t fmt;
+	unsigned long no;
 	char *slash;
 	int ch;
 
@@ -665,12 +841,65 @@ int main(int argc, char **argv)
 			if (errno || *slash)
 				errx(EXIT_FAILURE, "Invalid argument for -%c", ch);
 			break;
+		case 'H':
+			hideundef = true;
+			break;
+		case 's':
+			shortname = true;
+			break;
+		case 'x':
+			ctrformat = "%lx";
+			break;
+		case 'X':
+			ctrformat = "%#lx";
+			break;
 		case 'a':
 			allcpu = true;
 			break;
+		case 'd':
+			delta = true;
+			firstread = true;
+			break;
+		case 't':
+			errno = 0;
+			no = strtoul(optarg, &slash, 0);
+			if (errno)
+				errx(EXIT_FAILURE, "Invalid argument for -%c", ch);
+			switch (*slash) {
+			case 's':
+			case '\0':
+				timeout += no;
+				break;
+			case 'm':
+				timeout += no * 60;
+				break;
+			case 'h':
+				timeout += no * 60 * 60;
+				break;
+			case 'd':
+				timeout += no * 60 * 60 * 24;
+				break;
+			default:
+				errx(EXIT_FAILURE, "Invalid argument for -%c", ch);
+				break;
+			}
+			break;
+		case 'q':
+			quote_all = true;
+			break;
+		case 'f':
+			if (!util_fmt_name_to_type(optarg, &fmt))
+				errx(EXIT_FAILURE, "Supported formats:" FMT_TYPE_NAMES);
+			output_format = fmt;
+			break;
 		}
 	}
 
+	if (timeout && timeout < read_interval)
+		read_interval = timeout;
+	/* If no timeout specified, simply add zero */
+	loop_count += timeout / read_interval;
+
 	have_support();
 	if (!libcpumf_cpumcf_info(&cfvn, &csvn, &authorization))
 		return EXIT_FAILURE;
@@ -681,6 +910,11 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
+	for (unsigned int i = 0; i < ARRAY_SIZE(ctrname); ++i) {
+		ctrname[i].ccv = util_zalloc(max_possible_cpus * sizeof(unsigned long));
+		ctrname[i].ccvprv = util_zalloc(max_possible_cpus * sizeof(unsigned long));
+	}
+
 	if (optind >= argc) {
 		ch = do_it(NULL);
 	} else {
@@ -692,5 +926,6 @@ int main(int argc, char **argv)
 	}
 	free_counternames();
 	free(check);
+	free(ioctlbuffer);
 	return ch;
 }
