@@ -66,6 +66,12 @@ void ht_print_scroll_bar(int row_cnt, int row_start, int row_bar_start,
 			 int row_bar_bottom, int can_scroll_up,
 			 int can_scroll_down, int with_boder);
 
+/*
+ * util_fmt helper functions
+ */
+void ht_fmt_time(void);
+void ht_fmt_cpu_types(void);
+
 /*
  * Error Macros
  */
