@@ -27,16 +27,9 @@
 #define DF_S390_CPU_MAX		512
 #define DF_S390_MAGIC_BLK_ECKD	3
 #define DF_S390_DUMPER_MAGIC_SIZE	7
-#define DF_S390_DUMPER_MAGIC32		"ZECKD31"
-#define DF_S390_DUMPER_MAGIC64		"ZECKD64"
 #define DF_S390_DUMPER_MAGIC_EXT	"XECKD64"
-#define DF_S390_DUMPER_MAGIC32_FBA	"ZDFBA31"
-#define DF_S390_DUMPER_MAGIC64_FBA	"ZDFBA64"
 #define DF_S390_DUMPER_MAGIC_FBA_EXT	"XDFBA64"
-#define DF_S390_DUMPER_MAGIC_MV		"ZMULT64"
 #define DF_S390_DUMPER_MAGIC_MV_EXT	"XMULT64"
-#define OLD_DUMPER_HEX_INSTR1 "\x0d\x10\x47\xf0" /* BASR + 1st halfword of BC */
-#define OLD_DUMPER_HEX_INSTR2 "\x0d\xd0" /* BASR 13,0 */
 
 /*
  * Architecture of dumped system
