@@ -9,6 +9,10 @@
  * it under the terms of the MIT license. See LICENSE for details.
  */
 
+#include "lib/util_path.h"
+#include "lib/util_file.h"
+#include "lib/util_libc.h"
+
 #include "ipl_tools.h"
 
 static struct {
@@ -42,24 +46,23 @@ static const char *get_ipl_banner(int show_ipl)
 void print_nss(int show_ipl)
 {
 	char *dir = show_ipl ? "ipl" : "reipl/nss";
-	char *path_bootparms = show_ipl ? "/sys/firmware/ipl/parm" :
-		"/sys/firmware/reipl/nss/parm";
+	char *path_bootparms = util_path_sysfs("firmware/%s/parm", dir);
 
 	printf("%-12s nss\n", get_ipl_banner(show_ipl));
 	print_fw_str("Name:        %s\n", dir, "name");
 	if (access(path_bootparms, R_OK) == 0)
 		print_fw_str("Bootparms:   \"%s\"\n", dir, "parm");
+	free(path_bootparms);
 }
 
 void print_fcp(int show_ipl, int dump)
 {
 	char *dir = show_ipl ? "ipl" : "reipl/fcp";
-	char *path_bootparms = show_ipl ? "/sys/firmware/ipl/scp_data" :
-		"/sys/firmware/reipl/fcp/scp_data";
-	char *path_loadparm = show_ipl ? "/sys/firmware/ipl/loadparm" :
-		"/sys/firmware/reipl/fcp/loadparm";
-	char loadparm[9], loadparm_path[PATH_MAX];
-	char *path_reipl_clear = "/sys/firmware/reipl/fcp/clear";
+	char *path_bootparms = util_path_sysfs("firmware/%s/scp_data", dir);
+	char *path_loadparm = util_path_sysfs("firmware/%s/loadparm", dir);
+	char *path_reipl_clear = util_path_sysfs("firmware/reipl/fcp/clear");
+	char *path_secure_boot = util_path_sysfs("firmware/ipl/secure");
+	char *loadparm;
 
 	if (dump)
 		printf("%-12s fcp_dump\n", get_ipl_banner(show_ipl));
@@ -72,27 +75,31 @@ void print_fcp(int show_ipl, int dump)
 	print_fw_str("bootprog:    %s\n", dir, "bootprog");
 	print_fw_str("br_lba:      %s\n", dir, "br_lba");
 	if (access(path_loadparm, R_OK) == 0) {
-		sprintf(loadparm_path, "%s/%s", dir, "loadparm");
-		read_fw_str(loadparm, loadparm_path, sizeof(loadparm));
-		if (strcmp(loadparm, "        ") == 0)
-			loadparm[0] = 0;
+		loadparm = util_file_read_text_file(path_loadparm, 1);
+		util_strstrip(loadparm);
 		printf("Loadparm:    \"%s\"\n", loadparm);
+		free(loadparm);
 	}
 	if (access(path_bootparms, R_OK) == 0)
 		print_fw_str("Bootparms:   \"%s\"\n", dir, "scp_data");
 	if (!show_ipl && access(path_reipl_clear, R_OK) == 0)
 		print_fw_str("clear:       %s\n", dir, "clear");
+	if (access(path_secure_boot, R_OK) == 0)
+		print_fw_str("Secure boot: %s\n", "ipl", "secure");
+	free(path_bootparms);
+	free(path_loadparm);
+	free(path_reipl_clear);
+	free(path_secure_boot);
 }
 
 void print_nvme(int show_ipl, int dump)
 {
 	char *dir = show_ipl ? "ipl" : "reipl/nvme";
-	char *path_bootparms = show_ipl ? "/sys/firmware/ipl/scp_data" :
-		"/sys/firmware/reipl/nvme/scp_data";
-	char *path_loadparm = show_ipl ? "/sys/firmware/ipl/loadparm" :
-		"/sys/firmware/reipl/nvme/loadparm";
-	char loadparm[9], loadparm_path[PATH_MAX];
-	char *path_reipl_clear = "/sys/firmware/reipl/nvme/clear";
+	char *path_bootparms = util_path_sysfs("firmware/%s/scp_data", dir);
+	char *path_loadparm = util_path_sysfs("firmware/%s/loadparm", dir);
+	char *path_reipl_clear = util_path_sysfs("firmware/reipl/nvme/clear");
+	char *path_secure_boot = util_path_sysfs("firmware/ipl/secure");
+	char *loadparm;
 
 	if (dump)
 		printf("%-12s nvme_dump\n", get_ipl_banner(show_ipl));
@@ -104,49 +111,54 @@ void print_nvme(int show_ipl, int dump)
 	print_fw_str("bootprog:    %s\n", dir, "bootprog");
 	print_fw_str("br_lba:      %s\n", dir, "br_lba");
 	if (access(path_loadparm, R_OK) == 0) {
-		sprintf(loadparm_path, "%s/%s", dir, "loadparm");
-		read_fw_str(loadparm, loadparm_path, sizeof(loadparm));
-		if (strcmp(loadparm, "        ") == 0)
-			loadparm[0] = 0;
+		loadparm = util_file_read_text_file(path_loadparm, 1);
+		util_strstrip(loadparm);
 		printf("Loadparm:    \"%s\"\n", loadparm);
+		free(loadparm);
 	}
 	if (access(path_bootparms, R_OK) == 0)
 		print_fw_str("Bootparms:   \"%s\"\n", dir, "scp_data");
 	if (!show_ipl && access(path_reipl_clear, R_OK) == 0)
 		print_fw_str("clear:       %s\n", dir, "clear");
+	if (access(path_secure_boot, R_OK) == 0)
+		print_fw_str("Secure boot: %s\n", "ipl", "secure");
+	free(path_bootparms);
+	free(path_loadparm);
+	free(path_reipl_clear);
+	free(path_secure_boot);
 }
 
 void print_ccw(int show_ipl)
 {
-	char loadparm[9], loadparm_path[PATH_MAX];
 	char *dir = show_ipl ? "ipl" : "reipl/ccw";
-	char *path_loadparm = show_ipl ? "/sys/firmware/ipl/loadparm" :
-		"/sys/firmware/reipl/ccw/loadparm";
-	char *path_bootparms = show_ipl ? "/sys/firmware/ipl/parm" :
-		"/sys/firmware/reipl/ccw/parm";
-	char *path_reipl_clear = "/sys/firmware/reipl/ccw/clear";
+	char *path_loadparm = util_path_sysfs("firmware/%s/loadparm", dir);
+	char *path_bootparms = util_path_sysfs("firmware/%s/parm", dir);
+	char *path_reipl_clear = util_path_sysfs("firmware/reipl/ccw/clear");
+	char *loadparm;
 
 	printf("%-12s ccw\n", get_ipl_banner(show_ipl));
 	print_fw_str("Device:      %s\n", dir, "device");
 	if (access(path_loadparm, R_OK) == 0) {
-		sprintf(loadparm_path, "%s/%s", dir, "loadparm");
-		read_fw_str(loadparm, loadparm_path, sizeof(loadparm));
-		if (strcmp(loadparm, "        ") == 0)
-			loadparm[0] = 0;
+		loadparm = util_file_read_text_file(path_loadparm, 1);
+		util_strstrip(loadparm);
 		printf("Loadparm:    \"%s\"\n", loadparm);
+		free(loadparm);
 	}
 	if (access(path_bootparms, R_OK) == 0)
 		print_fw_str("Bootparms:   \"%s\"\n", dir, "parm");
 	if (!show_ipl && access(path_reipl_clear, R_OK) == 0)
 		print_fw_str("clear:       %s\n", dir, "clear");
+	free(path_loadparm);
+	free(path_bootparms);
+	free(path_reipl_clear);
 }
 
 void print_eckd(int show_ipl, const char *name)
 {
-	char *path_loadparm = show_ipl ? "/sys/firmware/ipl/loadparm" :
-		"/sys/firmware/reipl/eckd/loadparm";
 	char *dir = show_ipl ? "ipl" : "reipl/eckd";
-	char loadparm[9], loadparm_path[PATH_MAX];
+	char *path_loadparm = util_path_sysfs("firmware/%s/loadparm", dir);
+	char *path_secure_boot = util_path_sysfs("firmware/ipl/secure");
+	char *loadparm;
 
 	printf("%-12s %s\n", get_ipl_banner(show_ipl), name);
 
@@ -155,14 +167,17 @@ void print_eckd(int show_ipl, const char *name)
 	print_fw_str("br_chr:      %s\n", dir, "br_chr");
 	print_fw_str("Bootparm:    \"%s\"\n", dir, "scp_data");
 	if (access(path_loadparm, R_OK) == 0) {
-		sprintf(loadparm_path, "%s/%s", dir, "loadparm");
-		read_fw_str(loadparm, loadparm_path, sizeof(loadparm));
-		if (strcmp(loadparm, "        ") == 0)
-			loadparm[0] = 0;
+		loadparm = util_file_read_text_file(path_loadparm, 1);
+		util_strstrip(loadparm);
 		printf("Loadparm:    \"%s\"\n", loadparm);
+		free(loadparm);
 	}
 	if (!show_ipl)
 		print_fw_str("clear:       %s\n", dir, "clear");
+	if (access(path_secure_boot, R_OK) == 0)
+		print_fw_str("Secure boot: %s\n", "ipl", "secure");
+	free(path_loadparm);
+	free(path_secure_boot);
 }
 
 static void parse_lsreipl_options(int argc, char *argv[])
@@ -196,16 +211,14 @@ static void parse_lsreipl_options(int argc, char *argv[])
 
 void cmd_lsreipl(int argc, char *argv[])
 {
-	char reipl_type_str[1024];
+	char *reipl_type_str;
 
 	parse_lsreipl_options(argc, argv);
 
 	if (l.ipl_set)
-		read_fw_str(reipl_type_str, "ipl/ipl_type",
-			    sizeof(reipl_type_str));
+		reipl_type_str = read_fw_str("ipl/ipl_type");
 	else
-		read_fw_str(reipl_type_str, "reipl/reipl_type",
-			    sizeof(reipl_type_str));
+		reipl_type_str = read_fw_str("reipl/reipl_type");
 
 	if (strcmp(reipl_type_str, "fcp") == 0)
 		print_fcp(l.ipl_set, 0);
@@ -225,5 +238,6 @@ void cmd_lsreipl(int argc, char *argv[])
 	else
 		printf("%s: %s (unknown)\n", get_ipl_banner(l.ipl_set),
 		       reipl_type_str);
+	free(reipl_type_str);
 	exit(0);
 }
