@@ -55,7 +55,7 @@ pub enum Error {
     #[error("Decryption failed. Probably due to a GCM tag mismatch.")]
     GcmTagMismatch,
 
-    #[error("Invalid {0} user-data for signing provided. Max {} bytes allowed", .0.max())]
+    #[error("Invalid {0} user-data for signing provided. Max {max} bytes allowed", max=.0.max())]
     AsrcbInvSgnUserData(UserDataType),
 
     #[error("Unsupported user data signing key provided. Only EC(secp521r1) and RSA(2048 & 3072 bit) are supported")]
@@ -106,6 +106,20 @@ pub enum Error {
     )]
     AddDataMissing(&'static str),
 
+    #[error("An ASCII string was expected, but non-ASCII characters were received.")]
+    NonAscii,
+
+    #[error("Incorrect {what} for a {kind}. Is: {value}; expected: {exp}")]
+    RetrInvKey {
+        what: &'static str,
+        kind: String,
+        value: String,
+        exp: String,
+    },
+
+    #[error("Invalid data from OpenSSL")]
+    InvalSslData,
+
     // errors from other crates
     #[error(transparent)]
     PvCore(#[from] pv_core::Error),
