@@ -7,11 +7,11 @@ use std::{
     path::Path,
 };
 
-use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U64};
+use zerocopy::{BigEndian, FromBytes, Immutable, IntoBytes, U64};
 
 use crate::{
     macros::{bail_spec, file_error},
-    Error, FileAccessErrorType, FileIoErrorType, Result,
+    Error, FileAccessErrorType, Result,
 };
 
 /// Trait that describes bitflags, represented by `T`.
@@ -34,7 +34,7 @@ pub trait Flags<T>: From<T> + for<'a> From<&'a T> {
 ///
 /// Wraps an u64 to set/get individual bits
 #[repr(C)]
-#[derive(Debug, Clone, Copy, Default, AsBytes, FromZeroes, FromBytes)]
+#[derive(Debug, Clone, Copy, Default, IntoBytes, FromBytes, Eq, PartialEq, Immutable)]
 pub struct Msb0Flags64(U64<BigEndian>);
 impl Flags<u64> for Msb0Flags64 {
     #[track_caller]
@@ -82,7 +82,7 @@ impl From<Msb0Flags64> for u64 {
 ///
 /// Wraps an u64 to set/get individual bits
 #[repr(C)]
-#[derive(Debug, Clone, Copy, Default, AsBytes, FromZeroes, FromBytes)]
+#[derive(Debug, Clone, Copy, Default, IntoBytes, FromBytes, Immutable)]
 pub struct Lsb0Flags64(U64<BigEndian>);
 impl Flags<u64> for Lsb0Flags64 {
     #[track_caller]
@@ -262,6 +262,17 @@ pub fn read_exact_file<P: AsRef<Path>, const COUNT: usize>(
     Ok(buf)
 }
 
+/// Read content from a file as string and add context in case of an error
+///
+/// * `path` - Path to file
+/// * `ctx` - Error context string in case of an error
+///
+/// # Errors
+/// Passes through any kind of error `std::fs::read` produces
+pub fn read_file_string<P: AsRef<Path>>(path: P, ctx: &str) -> Result<String> {
+    std::fs::read_to_string(&path).map_err(|e| file_error!(Read, ctx, path, e))
+}
+
 /// Read content from a file and add context in case of an error
 ///
 /// * `path` - Path to file
@@ -284,12 +295,8 @@ pub fn read_file<P: AsRef<Path>>(path: P, ctx: &str) -> Result<Vec<u8>> {
 /// Passes through any kind of error `std::fs::read` produces
 pub fn read<R: Read, P: AsRef<Path>>(rd: &mut R, path: P, ctx: &str) -> Result<Vec<u8>> {
     let mut buf = vec![];
-    rd.read_to_end(&mut buf).map_err(|e| Error::FileIo {
-        ty: FileIoErrorType::Write,
-        ctx: ctx.to_string(),
-        path: path.as_ref().to_path_buf(),
-        source: e,
-    })?;
+    rd.read_to_end(&mut buf)
+        .map_err(|e| file_error!(Read, ctx, path, e))?;
     Ok(buf)
 }
 
@@ -302,12 +309,7 @@ pub fn read<R: Read, P: AsRef<Path>>(rd: &mut R, path: P, ctx: &str) -> Result<V
 /// # Errors
 /// Passes through any kind of error `std::fs::write` produces
 pub fn write_file<D: AsRef<[u8]>, P: AsRef<Path>>(path: P, data: D, ctx: &str) -> Result<()> {
-    std::fs::write(path.as_ref(), data.as_ref()).map_err(|e| Error::FileIo {
-        ty: FileIoErrorType::Write,
-        ctx: ctx.to_string(),
-        path: path.as_ref().to_path_buf(),
-        source: e,
-    })
+    std::fs::write(path.as_ref(), data.as_ref()).map_err(|e| file_error!(Write, ctx, path, e))
 }
 
 /// Write content to a [`std::io::Write`] and add context in case of an error
@@ -324,12 +326,8 @@ pub fn write<D: AsRef<[u8]>, P: AsRef<Path>, W: Write>(
     path: P,
     ctx: &str,
 ) -> Result<()> {
-    wr.write_all(data.as_ref()).map_err(|e| Error::FileIo {
-        ty: FileIoErrorType::Write,
-        ctx: ctx.to_string(),
-        path: path.as_ref().to_path_buf(),
-        source: e,
-    })
+    wr.write_all(data.as_ref())
+        .map_err(|e| file_error!(Write, ctx, path, e))
 }
 
 macro_rules! usize_to_ui {
@@ -355,6 +353,15 @@ usize_to_ui! {
 #[doc = r"u16"]
 => u16, to_u16}
 
+/// Converts the u8 slice into (lowercase) hexstring
+pub fn encode_hex<S: AsRef<[u8]>>(s: S) -> String {
+    let slice = s.as_ref();
+    let string = String::with_capacity(2 * slice.len());
+    slice
+        .iter()
+        .fold(string, |acc, e| acc + &format!("{e:02x}"))
+}
+
 /// Converts the hexstring into a byte vector.
 ///
 /// # Errors
@@ -504,6 +511,14 @@ mod tests {
     fn lsb_flags_unset_panic() {
         Lsb0Flags64::default().unset_bit(64)
     }
+
+    #[test]
+    fn encode_hex() {
+        let arr = [0x12, 0x34, 0x56, 0xac, 0xbe, 0xf0];
+        let exp = "123456acbef0";
+        assert_eq!(super::encode_hex(arr), exp);
+    }
+
     #[test]
     fn parse_hex() {
         let s = "123456acbef0";
