@@ -49,43 +49,32 @@ fn parse_flags(
     args: &CreateBootImageArgs,
 ) -> Result<(PlaintextControlFlagsV1, SecretControlFlagsV1)> {
     let lf = &args.legacy_flags;
+    macro_rules! flag_disabled {
+        ($cli_flag:expr, $control_flags:expr) => {
+            $cli_flag
+                .filter(|x| *x)
+                .and(Some(ControlFlagTrait::all_disabled($control_flags)))
+        };
+    }
+    macro_rules! flag_enabled {
+        ($cli_flag:expr, $control_flags:expr) => {
+            $cli_flag
+                .filter(|x| *x)
+                .and(Some(ControlFlagTrait::all_enabled($control_flags)))
+        };
+    }
+
     let plaintext_flags: Vec<FlagData<PcfV1>> = [
-        lf.disable_dump
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_disabled([PcfV1::AllowDumping]))),
-        lf.enable_dump
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_enabled([PcfV1::AllowDumping]))),
-        lf.disable_pckmo
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_disabled([
-                PcfV1::PckmoAes,
-                PcfV1::PckmoDeaTdea,
-                PcfV1::PckmoEcc,
-            ]))),
-        lf.enable_pckmo.filter(|x| *x).and(Some(PcfV1::all_enabled([
-            PcfV1::PckmoAes,
-            PcfV1::PckmoDeaTdea,
-            PcfV1::PckmoEcc,
-        ]))),
-        lf.disable_pckmo_hmac
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_disabled([PcfV1::PckmoHmac]))),
-        lf.enable_pckmo_hmac
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_enabled([PcfV1::PckmoHmac]))),
-        lf.disable_backup_keys
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_disabled([PcfV1::BackupTargetKeys]))),
-        lf.enable_backup_keys
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_enabled([PcfV1::BackupTargetKeys]))),
-        lf.disable_image_encryption
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_enabled([PcfV1::NoComponentEncryption]))),
-        lf.enable_image_encryption
-            .filter(|x| *x)
-            .and(Some(PcfV1::all_disabled([PcfV1::NoComponentEncryption]))),
+        flag_disabled!(lf.disable_dump, [PcfV1::AllowDumping]),
+        flag_enabled!(lf.enable_dump, [PcfV1::AllowDumping]),
+        flag_disabled!(lf.disable_pckmo, PlaintextControlFlagsV1::PCKMO),
+        flag_enabled!(lf.enable_pckmo, PlaintextControlFlagsV1::PCKMO),
+        flag_disabled!(lf.disable_pckmo_hmac, [PcfV1::PckmoHmac]),
+        flag_enabled!(lf.enable_pckmo_hmac, [PcfV1::PckmoHmac]),
+        flag_disabled!(lf.disable_backup_keys, [PcfV1::BackupTargetKeys]),
+        flag_enabled!(lf.enable_backup_keys, [PcfV1::BackupTargetKeys]),
+        flag_enabled!(lf.disable_image_encryption, [PcfV1::NoComponentEncryption]),
+        flag_disabled!(lf.enable_image_encryption, [PcfV1::NoComponentEncryption]),
     ]
     .into_iter()
     .flatten()
@@ -95,16 +84,16 @@ fn parse_flags(
     assert!(PlaintextControlFlagsV1::no_duplicates(&plaintext_flags));
 
     let secret_flags: Vec<FlagData<ScfV1>> = [
-        lf.disable_cck_extension_secret
-            .filter(|x| *x)
-            .and(Some(ScfV1::all_disabled([
-                ScfV1::CCKExtensionSecretEnforcment,
-            ]))),
-        lf.enable_cck_extension_secret
-            .filter(|x| *x)
-            .and(Some(ScfV1::all_enabled([
-                ScfV1::CCKExtensionSecretEnforcment,
-            ]))),
+        flag_disabled!(
+            lf.disable_cck_extension_secret,
+            [ScfV1::CckExtensionSecretEnforcement]
+        ),
+        flag_enabled!(
+            lf.enable_cck_extension_secret,
+            [ScfV1::CckExtensionSecretEnforcement]
+        ),
+        flag_disabled!(lf.disable_cck_update, [ScfV1::CckUpdateAllowed]),
+        flag_enabled!(lf.enable_cck_update, [ScfV1::CckUpdateAllowed]),
     ]
     .into_iter()
     .flatten()
@@ -219,3 +208,29 @@ pub fn create(opt: &CreateBootImageArgs) -> Result<OwnExitCode> {
     warn!("Successfully generated the Secure Execution image.");
     Ok(OwnExitCode::Success)
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    use crate::cli::CreateBootImageLegacyFlags;
+
+    #[test]
+    fn parse_flags() {
+        let args = CreateBootImageArgs {
+            legacy_flags: CreateBootImageLegacyFlags {
+                enable_dump: Some(true),
+                enable_cck_update: Some(true),
+                ..Default::default()
+            },
+            ..Default::default()
+        };
+        let parsed_flags = super::parse_flags(&args).expect("Failed to parse flags {args:?}");
+        let mut exp_pcf = Vec::from(PlaintextControlFlagsV1::PCKMO);
+        exp_pcf.push(PcfV1::AllowDumping);
+        let pcf = PlaintextControlFlagsV1::from_flags(PcfV1::all_enabled(exp_pcf));
+        assert_eq!(parsed_flags.0, pcf);
+        let exp_scf = vec![ScfV1::CckUpdateAllowed];
+        let scf = SecretControlFlagsV1::from_flags(ScfV1::all_enabled(exp_scf));
+        assert_eq!(parsed_flags.1, scf);
+    }
+}
