@@ -217,6 +217,16 @@ pub enum AddSecretType {
         #[arg(long = "type", value_name = "TYPE")]
         kind: RetrieveableSecretInpKind,
     },
+
+    /// Update customer communication key.
+    ///
+    /// Insert a customer communication key into a guest.
+    #[command(visible_alias = "cck")]
+    UpdateCck {
+        /// Use CCK-FILE as new CCK.
+        #[arg(long, value_name = "CCK-FILE", value_hint = ValueHint::FilePath)]
+        secret: String,
+    },
 }
 
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug)]
@@ -228,10 +238,11 @@ pub enum RetrieveableSecretInpKind {
     /// Must be a plain byte file 128, 192, or 256 bit long.
     Aes,
     /// An AES-XTS key.
-    /// Must be a plain byte file 512, or 1024 bit long.
+    /// Must be a plain byte file 256, or 512 bit long.
     AesXts,
     /// A HMAC-SHA key.
-    /// Must be a plain byte file 512, or 1024 bit long.
+    /// Must be a plain byte file 512, or 1024 bit long. Special care is required when creating
+    /// HMAC-SHA keys. For more Information refer to the DESCRIPTION section of the man file.
     HmacSha,
     /// An elliptic curve private key.
     /// Must be a PEM or DER file.
@@ -261,6 +272,13 @@ pub struct AddSecretOpt {
     #[arg(value_name = "FILE", value_hint = ValueHint::FilePath,)]
     #[cfg(target_arch = "s390x")]
     pub input: String,
+
+    /// Force the addition of add-secret requests.
+    ///
+    /// Add an add-secret request even if there is already a secret with the same ID in the secret
+    /// store.
+    #[arg(short, long)]
+    pub force: bool,
 }
 
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug, Default)]
@@ -319,8 +337,10 @@ pub struct RetrSecretOptions {
     /// Specify the secret ID to be retrieved.
     ///
     /// Input type depends on '--inform'. If `yaml` (default) is specified, it must be a yaml
-    /// created by the create subcommand of this tool. If `hex` is specified, it must be a hex
-    /// 32-byte unsigned big endian number string. Leading zeros are required.
+    /// created by the create subcommand of this tool. If `hex` is specified, it must be a 32 byte
+    /// handle encodes in hexadecimal. Leading zeros are required. If there are multiple secrets in
+    /// the store with the same Id there are no guarantees on which specific secret is retrieved.
+    /// Use --inform=idx to make sure a specific secret is retrieved.
     #[cfg(target_arch = "s390x")]
     #[arg(value_name = "ID", value_hint = ValueHint::FilePath)]
     pub input: String,
@@ -350,6 +370,8 @@ pub enum RetrInpFmt {
     Hex,
     /// Use a name-string. Will hash it if no secret with the name found.
     Name,
+    /// Use the secret-index (base 10) instead of the secret-ID
+    Idx,
 }
 
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum, Debug, Default)]
@@ -462,6 +484,7 @@ mod test {
             vec!["pvsecret", "add"],
             vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "meta"],
             vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "association", "name" ],
+            vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "update-cck", "--secret", "abc"],
             // verify that arguments stay backwards compatible
             vec!["pvsecret", "create", "-k", "abc,cdef", "--hdr", "abc", "-o", "abc", "-C", "uuu,ggg", "--crl", "yyy,hhh", "--root-ca", "tttt",
                  "--extension-secret", "fff", "--cuid", "cuid", "--flags", "disable-dump", "meta"],
@@ -489,6 +512,7 @@ mod test {
             vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "--cuid", "abc", "--cuid_hex", "9",  "null"],
             vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "association"],
             vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "association", "name", "--output-secret", "secret", "--input-secret", "secret"],
+            vec!["pvsecret", "create", "-k", "abc", "--hdr", "abc", "-o", "abc", "--no-verify", "update-cck"],
             ];
         for arg in valid_args {
             let res = CliOptions::try_parse_from(&arg);
