@@ -43,6 +43,7 @@ readonly NETWORK_DEVS=$(cd /sys/class/net; ls -d */ 2>/dev/null |  sed 's/\///g'
 # distro info
 readonly OSPRETTY="$(cat /etc/os* 2>/dev/null | grep -m1 PRETTY_NAME | sed 's/\"//g')"
 readonly OS_NAME="${OSPRETTY##*=}"
+readonly PODMAN=$(if type podman >/dev/null 2>&1; then echo "YES"; else echo "NO"; fi)
 # The processor ID for the first processor
 readonly PROCESSORID="$(grep -E ".*processor 0:.*" /proc/cpuinfo | \
 	sed 's/.*identification[[:space:]]*\=[[:space:]]*\([[:alnum:]]*\).*/\1/g')"
@@ -169,7 +170,7 @@ $(cat /proc/sysinfo | grep 'Name')
 Kernel version        = ${KERNEL_INFO}
 OS version / distro   = ${OS_NAME}
 KVM host              = ${KVM}
-container host        Kubernetes: ${KUBERNETES} - docker: ${DOCKER}
+container host        Kubernetes: ${KUBERNETES} - docker: ${DOCKER} - podman: ${PODMAN}
 
 Current user          = $(whoami) (must be root for data collection)
 Date and time         = $(date)
@@ -269,6 +270,7 @@ readonly OUTPUT_FILE_LSOF="${WORKPATH}open_files.out"
 readonly OUTPUT_FILE_NETWORK="${WORKPATH}network.out"
 readonly OUTPUT_FILE_NVME="${WORKPATH}runtime.out"
 readonly OUTPUT_FILE_OVS="${WORKPATH}network.out"
+readonly OUTPUT_FILE_PODMAN="${WORKPATH}podman_runtime.out"
 readonly OUTPUT_FILE_ISW="${WORKPATH}installed_sw.out"
 readonly OUTPUT_FILE_TC="${WORKPATH}network.out"
 readonly OUTPUT_FILE_VMCMD="${WORKPATH}zvm_runtime.out"
@@ -368,6 +370,7 @@ LOGFILES="\
   /var/log/anaconda.*\
   /var/log/audit\
   /var/log/boot*\
+  /var/log/crash/*/vmcore-dmesg.txt\
   /var/log/cron*\
   /var/log/dmesg*\
   /var/log/dnf.*\
@@ -401,10 +404,12 @@ CONFIGFILES="\
   /etc/*release\
   /etc/anacrontab\
   /etc/apparmor.d\
+  /etc/audit\
   /etc/auto.*\
   /etc/cmdline\
   /etc/cmdline.d\
   /etc/conf.d\
+  /etc/containers\
   /etc/cron.*\
   /etc/crontab\
   /etc/crypttab\
@@ -441,6 +446,7 @@ CONFIGFILES="\
   /etc/rc.local\
   /etc/resolv.*\
   /etc/rsyslog.d\
+  /etc/selinux\
   /etc/ssl/openssl.conf\
   /etc/ssl/openssl.cnf\
   /etc/sysconfig\
@@ -472,6 +478,10 @@ CONFIGFILES="\
 CMDS="uname -a\
   :uptime\
   :timedatectl\
+  :chronyc sources\
+  :chronyc sourcestats\
+  :chronyc ntpdata\
+  :chronyc tracking\
   :zhypinfo\
   :last\
   :lscpu -ae\
@@ -511,9 +521,9 @@ CMDS="${CMDS}\
 #   command groups itself have an intentional topical grouping and order
 CMDS="${CMDS}\
   :lsblk\
+  :lsblk -o +FSTYPE,UUID\
   :lsdasd\
   :lsdasd -u\
-  :lvdisplay\
   :pvpath -qa\
   :dmsetup ls\
   :dmsetup ls --tree\
@@ -540,6 +550,12 @@ CMDS="${CMDS}\
   :lszfcp -V\
   :ziorep_config -ADM\
   :lsscsi\
+  :fdisk -l\
+  :lvmconfig\
+  :lvdisplay -m\
+  :vgs -v\
+  :pvs -v\
+  :lvs -v\
   "
 
 # crypto specific commands
@@ -581,6 +597,7 @@ CMDS="${CMDS}\
 # commands with separate output file
 CMDS="${CMDS}\
   :apt list >> '${OUTPUT_FILE_ISW}'\
+  :snap list --all >> '${OUTPUT_FILE_ISW}'\
   :coredumpctl && coredumpctl info -o ${OUTPUT_FILE_COREDUMPCTL}\
   :journalctl --all --no-pager --lines=100000 --output=short-precise\
    >> '${OUTPUT_FILE_JOURNALCTL}'\
@@ -603,14 +620,16 @@ NETWORK_CMDS="ip -br a\
   :ip route list table all\
   :ip rule list\
   :ipcs -a\
-  :iptables -L\
+  :iptables -L # about to be replaced by nftables\
   :lsqeth\
   :netstat -pantu\
   :netstat -s\
+  :nft list tables # is replacing iptables\
   :nm-tool\
   :nstat -az\
   :openssl engine # deprecated in OpenSSL 3.n\
   :openssl list --providers 2>/dev/null # redirect help on older systems\
+  :opticsmon --module-info\
   :route -n\
   "
 
@@ -623,6 +642,17 @@ DOCKER_CMDS="docker version\
   :docker stats --no-stream\
   :systemctl status docker.service\
   "
+########################################
+PODMAN_CMDS="podman version\
+  :podman info\
+  :podman images\
+  :podman network ls\
+  :podman ps -a\
+  :podman stats --no-stream\
+  :podman pod stats -a --no-stream\
+  :ls -l /var/lib/containers/storage/overlay/* # list of files is sufficent\
+  :ls -l /var/log/crio/pods/*/*\
+  "
 
 ########################################
 KUBERNETES_CMDS="kubectl version\
@@ -1088,16 +1118,16 @@ collect_tc() {
 
 	if type tc >/dev/null; then
 		if test -n "${NETWORK_DEVS}"; then
-			pr_collect_output "Trafic Control"
+			pr_collect_output "Traffic Control"
 			for network_device in ${NETWORK_DEVS}; do
 				call_run_command "tc -s qdisc show dev ${network_device}" \
 					"${OUTPUT_FILE_TC}"
 			done
 		else
-			pr_skip "Trafic Control: no devices"
+			pr_skip "Traffic Control: no devices"
 		fi
 	else
-		pr_skip "Trafic Control: tc not available"
+		pr_skip "Traffic Control: tc not available"
 	fi
 }
 
@@ -1164,43 +1194,37 @@ collect_ovs() {
 ########################################
 collect_container() {
 	local container_list
-	local network_list
 	local item
 
 	# check if container environment exists
-	if test "x${DOCKER}" = "xYES" || test "x${KUBERNETES}" = "xYES"; then
+	if test "x${DOCKER}" = "xYES" || test "x${KUBERNETES}" = "xYES" || test "x${PODMAN}" = "xYES"; then
 		pr_collect_output "container host"
 	else
 		pr_skip "container host: not found"
 	fi
 
 	# for docker command exists
-	if [ "x${DOCKER}" = "xYES" ]; then
+	if test  "x${DOCKER}" = "xYES"; then
 		pr_syslog_stdout " docker ..."
-		container_list=$(docker ps -qa)
-		network_list=$(docker network ls -q)
 		IFS=:
 		for item in ${DOCKER_CMDS}; do
 			IFS=${IFS_ORI} call_run_command "${item}" "${OUTPUT_FILE_DOCKER}"
 		done
 		IFS="${IFS_ORI}"
+	fi
 
-		if test -n "${container_list}"; then
-			for item in ${container_list}; do
-				call_run_command "docker inspect ${item}" "${OUTPUT_FILE_DOCKER}"
-			done
-		fi
-
-		if test -n "${network_list}"; then
-			for item in ${network_list}; do
-				call_run_command "docker network inspect ${item}" \
-					"${OUTPUT_FILE_DOCKER}"
-			done
-		fi
+	# for podman command exists
+	if test  "x${PODMAN}" = "xYES"; then
+		pr_syslog_stdout " podman ..."
+		IFS=:
+		for item in ${PODMAN_CMDS}; do
+			IFS=${IFS_ORI} call_run_command "${item}" "${OUTPUT_FILE_PODMAN}"
+		done
+		IFS="${IFS_ORI}"
 	fi
 
 	# for kubectl command exists
-	if [ "x${KUBERNETES}" = "xYES" ]; then
+	if test "x${KUBERNETES}" = "xYES"; then
 		pr_syslog_stdout " Kubernetes ..."
 		container_list=$(kubectl top pod | grep -v "MEMORY(bytes)" | cut -d' ' -f1)
 
