@@ -16,10 +16,12 @@
 struct device;
 struct util_list;
 
-bool udev_ccwgroup_exists(const char *, const char *);
-exit_code_t udev_ccwgroup_read_device(struct device *);
-exit_code_t udev_ccwgroup_write_device(struct device *);
-void udev_ccwgroup_add_device_ids(const char *, struct util_list *);
-exit_code_t udev_ccwgroup_remove_rule(const char *, const char *);
+bool udev_ccwgroup_exists(const char *type, const char *id, bool autoconf);
+exit_code_t udev_ccwgroup_read_device(struct device *dev, bool autoconf);
+exit_code_t udev_ccwgroup_write_device(struct device *dev, bool autoconf);
+void udev_ccwgroup_add_device_ids(const char *type, struct util_list *list,
+				  bool autoconf);
+exit_code_t udev_ccwgroup_remove_rule(const char *type, const char *id,
+				      bool autoconf);
 
 #endif /* UDEV_CCWGROUP_H */
