@@ -26,6 +26,8 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "lib/util_file.h"
+
 #include "dasd.h"
 #include "device.h"
 #include "devtype.h"
@@ -97,71 +99,6 @@ static void dryrun_end_data(void)
 		fprintf(fd, "%c", DRYRUN_DATA_END);
 }
 
-#define READ_CHUNK_SIZE		4096
-
-/* Read all data from @fd and return address of resulting buffer in
- * @buffer_ptr. If @size_ptr is non-zero, use it to store the size of the
- * resulting buffer. Return %EXIT_OK on success. */
-exit_code_t misc_read_fd(FILE *fd, void **buffer_ptr, size_t *size_ptr)
-{
-	char *buffer = NULL;
-	size_t done = 0;
-
-	while (!feof(fd)) {
-		buffer = realloc(buffer, done + READ_CHUNK_SIZE);
-		if (!buffer)
-			oom();
-		done += fread(&buffer[done], 1, READ_CHUNK_SIZE, fd);
-		if (ferror(fd)) {
-			free(buffer);
-			return EXIT_RUNTIME_ERROR;
-		}
-	}
-
-	buffer = realloc(buffer, done);
-	if (!buffer && done > 0)
-		oom();
-
-	*buffer_ptr = buffer;
-	if (size_ptr)
-		*size_ptr = done;
-
-	return EXIT_OK;
-}
-
-/* Read text from @fd and return resulting NULL-terminated text buffer.
- * If @chomp is non-zero, remove trailing newline character. Return %NULL
- * on error or when unprintable characters are read. */
-static char *read_fd(FILE *fd, int chomp)
-{
-	char *buffer;
-	size_t done, i;
-
-	if (misc_read_fd(fd, (void **) &buffer, &done))
-		return NULL;
-
-	/* Check if this is a text file at all (required to filter out
-	 * binary sysfs attributes). */
-	for (i = 0; i < done; i++) {
-		if (!isgraph(buffer[i]) && !isspace(buffer[i])) {
-			free(buffer);
-			return NULL;
-		}
-	}
-
-	/* Remove trailing new-line character if requested. */
-	if (chomp && done > 0 && buffer[done - 1] == '\n')
-		done--;
-
-	/* NULL-terminate. */
-	buffer = realloc(buffer, done + 1);
-	if (!buffer)
-		oom();
-	buffer[done] = 0;
-
-	return buffer;
-}
-
 static int count_newline(const char *str)
 {
 	int i, newline;
@@ -182,7 +119,7 @@ static void dryrun_print(void)
 	int cmd, newline;
 
 	fseek(dryrun_file, 0, SEEK_SET);
-	txt = read_fd(dryrun_file, 0);
+	txt = util_file_read_fd(dryrun_file, 0);
 
 	if (!txt)
 		return;
@@ -979,7 +916,7 @@ char *misc_read_text_file(const char *path, int chomp, err_t err)
 	if (!fd)
 		goto out;
 
-	buffer = read_fd(fd, chomp);
+	buffer = util_file_read_fd(fd, chomp);
 	misc_fclose(fd);
 
 out:
@@ -1003,7 +940,7 @@ char *misc_read_cmd_output(const char *cmd, int chomp, err_t err)
 	if (!fd)
 		goto out;
 
-	buffer = read_fd(fd, chomp);
+	buffer = util_file_read_fd(fd, chomp);
 	pclose(fd);
 
 out:
@@ -1027,7 +964,7 @@ char *config_read_cmd_output(const char *cmd, int chomp, err_t err)
 	if (!fd)
 		goto out;
 
-	buffer = read_fd(fd, chomp);
+	buffer = util_file_read_fd(fd, chomp);
 	misc_pclose(fd);
 
 out:
@@ -1758,3 +1695,20 @@ char *misc_strrstr(const char *haystack, const char *needle)
 
 	return result;
 }
+
+/* Check if the site mentioned is valid */
+bool is_valid_site(const char *str)
+{
+	char *ptr = NULL;
+	long site_id;
+
+	site_id = strtol(str, &ptr, 10);
+
+	if (*ptr)
+		return false;
+
+	if (site_id < 0 || site_id >= NUM_USER_SITES)
+		return false;
+
+	return true;
+}
