@@ -30,7 +30,6 @@ static struct dfi *dfi_vec[] = {
 	&dfi_s390tape,
 	&dfi_devmem,
 	&dfi_s390mv_ext,
-	&dfi_s390mv,
 	&dfi_s390_ext,
 	&dfi_s390,
 	&dfi_lkcd,
@@ -68,7 +67,6 @@ struct attr {
 	struct timeval		*time_end;
 	u64			*cpu_id;
 	u64			*mem_size_real;
-	enum dfi_arch		*build_arch;
 	unsigned int		*vol_nr;
 	u32			*real_cpu_cnt;
 	struct new_utsname	*utsname;
@@ -82,7 +80,6 @@ struct attr {
  * File local static data
  */
 static struct {
-	enum dfi_arch	arch;
 	struct attr	attr;
 	struct cpus	cpus;
 	struct dfi	*dfi;
@@ -110,11 +107,38 @@ static void date_print(void)
 	}
 }
 
+/*
+ * Obtain online cpu count based on the lowcore_ptr entries
+ */
+static unsigned int lowcore_cpu_count(void)
+{
+	unsigned long *cpu_lowcore;
+	unsigned long ptr, length;
+	unsigned int i, count = 0;
+
+	if (dfi_vmcoreinfo_symbol(&ptr, "lowcore_ptr"))
+		return 0;
+	if (dfi_vmcoreinfo_length(&length, "lowcore_ptr") || length == 0)
+		return 0;
+	cpu_lowcore = zg_alloc(length * sizeof(*cpu_lowcore));
+	if (dfi_mem_virt_read(dfi_vm_vtop(ptr), cpu_lowcore, length * sizeof(*cpu_lowcore)) == 0) {
+		for (i = 0; i < length; i++) {
+			if (cpu_lowcore[i])
+				count++;
+		}
+		util_log_print(UTIL_LOG_INFO, "DFI lowcore_ptr entries count: %lu\n", count);
+	}
+	zg_free(cpu_lowcore);
+	return count;
+}
+
 /*
  * Print dump information (--info option)
  */
 void dfi_info_print(void)
 {
+	const unsigned int online_cpu_cnt = l.cpus.cnt ? l.cpus.cnt : lowcore_cpu_count();
+
 	STDERR("General dump info:\n");
 	STDERR("  Dump format........: %s\n", l.dfi->name);
 	if (l.attr.dfi_version)
@@ -131,12 +155,9 @@ void dfi_info_print(void)
 	}
 	if (l.attr.vol_nr)
 		STDERR("  Volume number......: %u\n", *l.attr.vol_nr);
-	if (l.attr.build_arch)
-		STDERR("  Build arch.........: %s\n",
-		       dfi_arch_str(*l.attr.build_arch));
-	STDERR("  System arch........: %s\n", dfi_arch_str(l.arch));
-	if (l.cpus.cnt)
-		STDERR("  CPU count (online).: %u\n", l.cpus.cnt);
+	STDERR("  System arch........: %s\n", dfi_arch_str(DFI_ARCH_64));
+	if (online_cpu_cnt)
+		STDERR("  CPU count (online).: %u\n", online_cpu_cnt);
 	if (l.attr.real_cpu_cnt)
 		STDERR("  CPU count (real)...: %u\n", *l.attr.real_cpu_cnt);
 	if (dfi_mem_range())
@@ -241,22 +262,6 @@ int dfi_cpu_content_fac_check(int flags)
 	return l.cpus.fac & flags;
 }
 
-/*
- * Set DFI architecture
- */
-void dfi_arch_set(enum dfi_arch arch)
-{
-	l.arch = arch;
-}
-
-/*
- * Return DFI architecture
- */
-enum dfi_arch dfi_arch(void)
-{
-	return l.arch;
-}
-
 /*
  * Return DFI CPU list
  */
@@ -299,8 +304,6 @@ const char *dfi_arch_str(enum dfi_arch arch)
 		return "s390 (32 bit)";
 	case DFI_ARCH_64:
 		return "s390x (64 bit)";
-	case DFI_ARCH_UNKNOWN:
-		return "unknown";
 	}
 	ABORT("dfi_arch_str: Invalid dfi arch: %d", arch);
 	return NULL; /* UNREACHABLE */
@@ -454,20 +457,6 @@ u32 *dfi_attr_zlib_entsize(void)
 	return l.attr.zlib_entsize;
 }
 
-/*
- * Attribute: Build architecture
- */
-void dfi_attr_build_arch_set(enum dfi_arch build_arch)
-{
-	l.attr.build_arch = zg_alloc(sizeof(*l.attr.build_arch));
-	*l.attr.build_arch = build_arch;
-}
-
-enum dfi_arch *dfi_attr_build_arch(void)
-{
-	return l.attr.build_arch;
-}
-
 /*
  * Attribute: Real CPU count
  */
@@ -482,48 +471,6 @@ unsigned int *dfi_attr_real_cpu_cnt(void)
 	return l.attr.real_cpu_cnt;
 }
 
-/*
- * Convert 32 bit CPU register set to 64 bit
- */
-static void cpu_32_to_64(struct dfi_cpu *cpu_64, struct dfi_cpu_32 *cpu_32)
-{
-	int i;
-
-	for (i = 0; i < 16; i++) {
-		cpu_64->gprs[i] = cpu_32->gprs[i];
-		cpu_64->ctrs[i] = cpu_32->ctrs[i];
-		cpu_64->acrs[i] = cpu_32->acrs[i];
-		if (i < 4)
-			cpu_64->fprs[i] = cpu_32->fprs[i];
-	}
-	cpu_64->psw[0] = cpu_32->psw[0];
-	cpu_64->psw[1] = cpu_32->psw[1];
-	cpu_64->prefix = cpu_32->prefix;
-	cpu_64->timer = cpu_32->timer;
-	cpu_64->todcmp = cpu_32->todcmp;
-}
-
-/*
- * Convert 64 bit CPU register set to 32 bit
- */
-void dfi_cpu_64_to_32(struct dfi_cpu_32 *cpu_32, struct dfi_cpu *cpu_64)
-{
-	int i;
-
-	for (i = 0; i < 16; i++) {
-		cpu_32->gprs[i] = (u32) cpu_64->gprs[i];
-		cpu_32->ctrs[i] = (u32) cpu_64->ctrs[i];
-		cpu_32->acrs[i] = (u32) cpu_64->acrs[i];
-		if (i < 4)
-			cpu_32->fprs[i] = (u32) cpu_64->fprs[i];
-	}
-	cpu_32->psw[0] = (u32) cpu_64->psw[0];
-	cpu_32->psw[1] = (u32) cpu_64->psw[1];
-	cpu_32->prefix = cpu_64->prefix;
-	cpu_32->timer = cpu_64->timer;
-	cpu_32->todcmp = cpu_64->todcmp;
-}
-
 /*
  * Copy 64 bit lowcore to internal register set
  */
@@ -555,21 +502,6 @@ static void lc2cpu_64(struct dfi_cpu *cpu, struct dfi_lowcore_64 *lc)
 	dfi_cpu_content_fac_add(DFI_CPU_CONTENT_FAC_VX);
 }
 
-/*
- * Copy 32 bit lowcore to internal 32 bit cpu
- */
-static void lc2cpu_32(struct dfi_cpu_32 *cpu, struct dfi_lowcore_32 *lc)
-{
-	memcpy(&cpu->gprs, lc->gpregs_save_area, sizeof(cpu->gprs));
-	memcpy(&cpu->ctrs, lc->cregs_save_area, sizeof(cpu->ctrs));
-	memcpy(&cpu->acrs, lc->access_regs_save_area, sizeof(cpu->acrs));
-	memcpy(&cpu->fprs, lc->floating_pt_save_area, sizeof(cpu->fprs));
-	memcpy(&cpu->psw, lc->st_status_fixed_logout, sizeof(cpu->psw));
-	memcpy(&cpu->prefix, &lc->prefixreg_save_area, sizeof(cpu->prefix));
-	memcpy(&cpu->timer, lc->timer_save_area, sizeof(cpu->timer));
-	memcpy(&cpu->todcmp, lc->clock_comp_save_area, sizeof(cpu->todcmp));
-}
-
 /*
  * Initialize and add a new CPU with given lowcore pointer
  *
@@ -579,6 +511,7 @@ static void lc2cpu_32(struct dfi_cpu_32 *cpu, struct dfi_lowcore_32 *lc)
 int dfi_cpu_add_from_lc(u32 lc_addr)
 {
 	struct dfi_cpu *cpu = dfi_cpu_alloc();
+	struct dfi_lowcore_64 lc;
 
 	cpu->cpu_id = l.cpus.cnt;
 	switch (l.cpus.content) {
@@ -586,19 +519,9 @@ int dfi_cpu_add_from_lc(u32 lc_addr)
 		cpu->prefix = lc_addr;
 		break;
 	case DFI_CPU_CONTENT_ALL:
-		if (l.arch == DFI_ARCH_32) {
-			struct dfi_cpu_32 cpu_32;
-			struct dfi_lowcore_32 lc;
-			if (dfi_mem_virt_read(lc_addr, &lc, sizeof(lc)))
-				return -EINVAL;
-			lc2cpu_32(&cpu_32, &lc);
-			cpu_32_to_64(cpu, &cpu_32);
-		} else {
-			struct dfi_lowcore_64 lc;
-			if (dfi_mem_virt_read(lc_addr, &lc, sizeof(lc)))
-				return -EINVAL;
-			lc2cpu_64(cpu, &lc);
-		}
+		if (dfi_mem_virt_read(lc_addr, &lc, sizeof(lc)))
+			return -EINVAL;
+		lc2cpu_64(cpu, &lc);
 		break;
 	case DFI_CPU_CONTENT_NONE:
 		ABORT("dfi_cpu_add_from_lc() called for CONTENT_NONE");
@@ -614,8 +537,6 @@ int dfi_cpu_lc_has_vx_sa(void *_lc)
 {
 	struct dfi_lowcore_64 *lc = _lc;
 
-	if (l.arch == DFI_ARCH_32)
-		return 0;
 	if (lc->vector_save_area_addr == 0)
 		return 0;
 	if (lc->vector_save_area_addr % 1024 != 0)
@@ -701,7 +622,6 @@ static void kdump_select_prod_init(void)
 		ERR_EXIT("The \"--select\" option is not possible with this "
 			 "dump");
 	attr_init();
-	dfi_arch_set(DFI_ARCH_64);
 	dfi_cpu_info_init(DFI_CPU_CONTENT_NONE);
 	if (dfi_vmcoreinfo_symbol(&ptr, "lowcore_ptr"))
 		return;
@@ -798,7 +718,6 @@ int dfi_init(void)
 
 	util_log_print(UTIL_LOG_TRACE, "DFI initialization\n");
 
-	l.arch = DFI_ARCH_UNKNOWN;
 	rc = dfi_mem_chunk_init();
 	if (rc)
 		return rc;
