@@ -324,6 +324,14 @@ void dfi_mem_chunk_read_zero(struct dfi_mem_chunk *UNUSED(mem_chunk),
 	memset(buf, 0, cnt);
 }
 
+/*
+ * Sort memory chunks by start value
+ */
+void dfi_mem_chunk_sort(void)
+{
+	util_list_sort(&l.mem_virt.chunk_list, mem_chunk_cmp_fn, NULL);
+}
+
 /*
  * Return mem_chunk list head
  */
@@ -350,6 +358,14 @@ u64 dfi_mem_range(void)
 	return l.mem_virt.end_addr - l.mem_virt.start_addr + 1;
 }
 
+/*
+ * Return maximum memory address
+ */
+u64 dfi_mem_end(void)
+{
+	return l.mem_virt.end_addr;
+}
+
 /*
  * Is memory range valid?
  */
