@@ -3,7 +3,7 @@
  *
  * Utility classes to print framsets
  *
- * Copyright IBM Corp. 2008, 2017
+ * Copyright IBM Corp. 2008, 2024
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -309,6 +309,22 @@ void PhysAdapterPrinter::print_phys_adpt(FILE *fp, __u32 host_id, int *rc)
 		fprintf(fp, "%3x", chpid);
 }
 
+void PhysAdapterPrinter::print_pchid(FILE *fp, __u32 host_id, int *rc)
+{
+	__u32 pchid = m_cfg->get_pchid_by_host_id(host_id, rc);
+
+	if (m_csv)
+		if (pchid != ZIOREP_PCHID_NA)
+			fprintf(fp, "%x", pchid);
+		else
+			fprintf(fp, "%s", "n/a");
+	else
+		if (pchid != ZIOREP_PCHID_NA)
+			fprintf(fp, "%04x", pchid);
+		else
+			fprintf(fp, "%s", "n/a ");
+}
+
 
 void PhysAdapterPrinter::print_utilization(FILE *fp,
 					  const struct abbrev_stat *stat,
@@ -361,10 +377,10 @@ void PhysAdapterPrinter::print_topline(FILE *fp)
 	if (m_csv)
 		fprintf(fp, "timestamp,aggregated,CHPID,adapter min %%,"
 			"adapter max %%,adapter avg %%,bus min %%,bus max %%,"
-			"bus avg %%,cpu min %%,cpu max %%,cpu avg %%\n");
+			"bus avg %%,cpu min %%,cpu max %%,cpu avg %%,PCHID\n");
 	else {
-		fprintf(fp, "CHP|adapter in %%-|--bus in %%---|--cpu in %%---|\n");
-		fprintf(fp, " ID min max   avg min max   avg min max   avg\n");
+		fprintf(fp, "PCH |CHP|adapter in %%-|--bus in %%---|--cpu in %%---|\n");
+		fprintf(fp, " ID   ID min max   avg min max   avg min max   avg\n");
 	}
 }
 
@@ -387,6 +403,12 @@ int PhysAdapterPrinter::print_frame(FILE *fp,
 				// print timestamp for every line in CSV mode
 				timestamp_printed = true;
 		}
+		if (!m_csv) {
+			print_pchid(fp, *i, &lrc);
+			if (lrc)
+				return -1;
+			print_delimiter(fp);
+		}
 		print_phys_adpt(fp, *i, &lrc);
 		if (lrc)
 			return -1;
@@ -402,6 +424,12 @@ int PhysAdapterPrinter::print_frame(FILE *fp,
 		print_utilization(fp, &util->stats.cpu,
 				  util->stats.count,
 				  util->valid);
+		if (m_csv) {
+			print_delimiter(fp);
+			print_pchid(fp, *i, &lrc);
+			if (lrc)
+				return -1;
+		}
 		fputc('\n', fp);
 	}
 
@@ -423,10 +451,11 @@ void VirtAdapterPrinter::print_virt_adpt(FILE *fp, __u32 devno,
 		fprintf(fp, "%x,%x.%x.%04x",
 			       m_cfg->get_chpid_by_devno(devno, rc),
 			       ZIOREP_BUSID_UNPACKED(devno));
-	else
-		fprintf(fp, "%3x/%x.%x.%04x",
-			       m_cfg->get_chpid_by_devno(devno, rc),
-			       ZIOREP_BUSID_UNPACKED(devno));
+	else {
+		fprintf(fp, "%x.%x.%04x", ZIOREP_BUSID_UNPACKED(devno));
+		print_delimiter(fp);
+		fprintf(fp, "%3x", m_cfg->get_chpid_by_devno(devno, rc));
+	}
 }
 
 void VirtAdapterPrinter::print_queue_fill(FILE *fp,
@@ -478,21 +507,22 @@ void VirtAdapterPrinter::print_failures(FILE *fp, const struct ioerr_cnt *cnt)
 }
 
 void VirtAdapterPrinter::print_throughput(FILE *fp,
-					 const struct blkiomon_stat *stat)
+					 const struct blkiomon_stat *stat,
+					 const __u64 interval)
 {
 	double tmp;
 
-	if (!stat || stat->d2c_r.num <= 0 || stat->size_r.num <= 0)
+	if (!stat || interval == 0 || stat->size_r.num <= 0)
 		tmp = 0;
 	else
-		tmp = stat->size_r.sum/(double)stat->d2c_r.sum;
+		tmp = calc_avg(stat->size_r.sum, interval);
 	print_delimiter(fp);
 	print_abbrev_num(fp, tmp);
 
-	if (!stat || stat->d2c_w.num <= 0 || stat->size_w.num <= 0)
+	if (!stat || interval == 0 || stat->size_w.num <= 0)
 		tmp = 0;
 	else
-		tmp = stat->size_w.sum/(double)stat->d2c_w.sum;
+		tmp = calc_avg(stat->size_w.sum, interval);
 	print_delimiter(fp);
 	print_abbrev_num(fp, tmp);
 }
@@ -519,8 +549,8 @@ void VirtAdapterPrinter::print_topline(FILE *fp)
 	if (m_csv)
 		fprintf(fp, "timestamp,aggregated,CHPID,Bus-ID,qdio utilization max %%,qdio utilization avg %%,queue full,fail erc,throughput read / MS/s,throughput write / MS/s,I/O requests read,I/O requests write\n");
 	else {
-		fprintf(fp, "CHP Bus-ID  |qdio util.%%|queu|fail|-thp in MB/s-|I/O reqs-|\n");
-		fprintf(fp, " ID            max   avg full  erc     rd    wrt   rd  wrt\n");
+		fprintf(fp, "Bus-ID  |CHP|qdio util.%%|queu|fail|-thp in MB/s-|I/O reqs-|\n");
+		fprintf(fp, "          ID   max   avg full  erc     rd    wrt   rd  wrt\n");
 	}
 }
 
@@ -536,6 +566,12 @@ int VirtAdapterPrinter::print_frame(FILE *fp,
 	const struct blkiomon_stat		*blk_stat;
 	const struct zfcpdd_dstat		*zfcp_stat;
 
+	/**
+	 * Receive Frameset interval and convert it
+	 * from seconds to microseconds
+	 */
+	__u64 interval = frameset.get_duration() * 1000000;
+
 	list<__u32> devnos;
 	devnos = ((StagedDeviceFilter*)&dev_filt)->get_filter_devnos();
 
@@ -557,7 +593,7 @@ int VirtAdapterPrinter::print_frame(FILE *fp,
 		print_queue_fill(fp, zfcp_stat, util);
 		print_queue_full(fp, util);
 		print_failures(fp, ioerr);
-		print_throughput(fp, blk_stat);
+		print_throughput(fp, blk_stat, interval);
 		print_num_requests(fp, blk_stat);
 		if (lrc) {
 			fprintf(stderr, "%s: Did not find matching data in"
@@ -631,9 +667,9 @@ void TrafficPrinter::print_topline_prefix1(FILE *fp)
 	switch (m_agg_crit) {
 	case none:
 		if (m_csv)
-			str = "WWPN,LUN";
+			str = "DEVBUSID,WWPN,LUN";
 		else
-			str = "       WWPN                LUN       ";
+			str = "DEVBUSID        WWPN                LUN       ";
 		break;
 	case chpid:
 		if (m_csv)
@@ -670,7 +706,7 @@ void TrafficPrinter::print_topline_prefix2(FILE *fp)
 
 	switch (m_agg_crit) {
 	case none:
-		str = "                                     ";
+		str = "                                              ";
 		break;
 	case chpid:
 		str = " ID";
@@ -698,7 +734,7 @@ void TrafficPrinter::print_topline_whitespace(FILE *fp)
 
 	switch (m_agg_crit) {
 	case none:
-		str = "                                     ";
+		str = "                                              ";
 		break;
 	case chpid:
 		str = "   ";
@@ -781,12 +817,16 @@ void TrafficPrinter::print_device_mp_mm(FILE *fp, __u32 mp_mm,
 void TrafficPrinter::print_device(FILE *fp, __u32 dev,
 				 const ConfigReader &cfg, int *rc)
 {
+	__u32 devno = cfg.get_devno_by_mm_internal(dev, rc);
+
+	print_device_devno(fp, devno);
+
 	if (m_csv)
-		fprintf(fp, "0x%016Lx,0x%016Lx",
+		fprintf(fp, ",0x%016Lx,0x%016Lx",
 			       (long long unsigned int)cfg.get_wwpn_by_mm_internal(dev, rc),
 			       (long long unsigned int)cfg.get_lun_by_mm_internal(dev, rc));
 	else
-		fprintf(fp, "0x%016Lx:0x%016Lx",
+		fprintf(fp, ":0x%016Lx:0x%016Lx",
 			       (long long unsigned int)cfg.get_wwpn_by_mm_internal(dev, rc),
 			       (long long unsigned int)cfg.get_lun_by_mm_internal(dev, rc));
 }
@@ -810,6 +850,12 @@ int TrafficPrinter::print_frame(FILE *fp, const Frameset &frameset,
 	const struct zfcpdd_dstat	*zfcp_stat = NULL;
 	const AggregationCollapser *agg_col;
 
+	/**
+	 * Receive Frameset interval and convert it
+	 * from seconds to microseconds
+	 */
+	__u64 interval = frameset.get_duration() * 1000000;
+
 	switch (m_agg_crit) {
 	case none:
 		get_device_list(lst_32, dev_filt);
@@ -847,7 +893,7 @@ int TrafficPrinter::print_frame(FILE *fp, const Frameset &frameset,
 			blk_stat = frameset.get_blkiomon_stat_by_wwpn(*i);
 			zfcp_stat = frameset.get_zfcpdd_stat_by_wwpn(*i);
 			print_device_wwpn(fp, *i);
-			print_data_row(fp, blk_stat, zfcp_stat);
+			print_data_row(fp, blk_stat, zfcp_stat, interval);
 		}
 	}
 	else if (m_agg_crit == all) {
@@ -856,7 +902,7 @@ int TrafficPrinter::print_frame(FILE *fp, const Frameset &frameset,
 		blk_stat = frameset.get_first_blkiomon_stat();
 		zfcp_stat = frameset.get_first_zfcpdd_stat();
 		print_device_all(fp);
-		print_data_row(fp, blk_stat, zfcp_stat);
+		print_data_row(fp, blk_stat, zfcp_stat, interval);
 	}
 	else {
 		for (list<__u32>::const_iterator i = lst_32.begin();
@@ -894,7 +940,7 @@ int TrafficPrinter::print_frame(FILE *fp, const Frameset &frameset,
 			}
 			if (rc )
 				return -1;
-			print_data_row(fp, blk_stat, zfcp_stat);
+			print_data_row(fp, blk_stat, zfcp_stat, interval);
 		}
 	}
 
@@ -930,7 +976,8 @@ void SummaryTrafficPrinter::print_topline(FILE *fp)
 }
 
 
-void SummaryTrafficPrinter::print_throughput(FILE *fp, const struct blkiomon_stat *stat)
+void SummaryTrafficPrinter::print_throughput(FILE *fp, const struct blkiomon_stat *stat,
+					     const __u64 interval)
 {
 	struct minmax thrp_data, total_size, total_latency;
 	double tmp;
@@ -968,7 +1015,7 @@ void SummaryTrafficPrinter::print_throughput(FILE *fp, const struct blkiomon_sta
 
 	tmp = 0;
 	if (stat && total_size.sum > 0)
-		tmp = calc_avg(total_size.sum, total_latency.sum);
+		tmp = calc_avg(total_size.sum, interval);
 	print_delimiter(fp);
 	print_abbrev_num(fp, tmp);
 
@@ -1101,14 +1148,15 @@ void SummaryTrafficPrinter::print_fabric_latency(FILE *fp, const struct zfcpdd_d
 
 void SummaryTrafficPrinter::print_data_row(FILE *fp,
 					const struct blkiomon_stat *blk_stat,
-					const struct zfcpdd_dstat *zfcp_stat)
+					const struct zfcpdd_dstat *zfcp_stat,
+					const __u64 interval)
 {
 	if (!blk_stat)
 		blk_stat = get_empty_blkiomon_stat();
 	if (!zfcp_stat)
 		zfcp_stat = get_empty_zfcpdd_dstat();
 
-	print_throughput(fp, blk_stat);
+	print_throughput(fp, blk_stat, interval);
 	print_request_stats(fp, blk_stat);
 	print_io_subsystem_latency(fp, blk_stat);
 	print_channel_latency(fp, zfcp_stat);
@@ -1177,7 +1225,8 @@ void DetailedTrafficPrinter::print_topline(FILE *fp)
 
 void DetailedTrafficPrinter::print_data_row(FILE *fp,
 			   const struct blkiomon_stat *blk_stat,
-			   const struct zfcpdd_dstat *zfcp_stat)
+			   const struct zfcpdd_dstat *zfcp_stat,
+			   const __u64 interval __attribute__ ((unused)))
 {
 	if (!blk_stat)
 		blk_stat = get_empty_blkiomon_stat();
