@@ -3,12 +3,14 @@
  *
  * Common ECKD dump I/O functions
  *
- * Copyright IBM Corp. 2013, 2018
+ * Copyright IBM Corp. 2013, 2023
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
  */
 
+#include "dump/s390_dump.h"
+
 #include "cio.h"
 #include "eckd2dump.h"
 #include "boot/error.h"
@@ -444,6 +446,38 @@ void readblock(unsigned long blk, unsigned long addr, unsigned long blk_count)
 	io_block(cmd, blk, addr, blk_count, 0, 1);
 }
 
+/*
+ * Write len/device.blk_size blocks of data at start address starting from
+ * given block and return the next free block number. The len value must be
+ * a multiple of a block size (4096 bytes).
+ */
+unsigned long write_addr_range(unsigned long blk, unsigned long start,
+			       unsigned long len, int print_progress)
+{
+	unsigned long addr, start_blk, blk_count, zero_page;
+
+	/* Verify the range is a multiple of a block size */
+	if (len % device.blk_size)
+		panic(EINTERNAL, "Invalid address range: [%016x, %016x]",
+		      start, start + len);
+	addr = start;
+	start_blk = blk;
+	while (addr < start + len) {
+		/* Remaining blocks to write */
+		blk_count = m2b(len) - (blk - start_blk);
+		blk_count = MIN(blk_count, eckd_blk_max);
+		zero_page = get_zeroed_page();
+		writeblock(blk, addr, blk_count, zero_page);
+		free_page(zero_page);
+		blk += blk_count;
+		addr += b2m(blk_count);
+		total_dump_size += b2m(blk_count);
+		if (print_progress)
+			progress_print(addr);
+	}
+	return blk;
+}
+
 /*
  * Write dump segment with the header to DASD and return the next free
  * block number
@@ -451,28 +485,16 @@ void readblock(unsigned long blk, unsigned long addr, unsigned long blk_count)
 unsigned long write_dump_segment(unsigned long blk,
 				 struct df_s390_dump_segm_hdr *segm)
 {
-	unsigned long addr, start_blk, blk_count, zero_page;
+	unsigned long zero_page;
 
 	/* Write the dump segment header itself (1 page) */
 	zero_page = get_zeroed_page();
-	writeblock(blk, (unsigned long)segm, m2b(PAGE_SIZE), zero_page);
+	writeblock(blk, (unsigned long)segm,
+		   m2b(sizeof(struct df_s390_dump_segm_hdr)), zero_page);
 	free_page(zero_page);
 	blk += m2b(PAGE_SIZE);
 	/* Write the dump segment */
-	addr = segm->start;
-	start_blk = blk;
-	while (addr < segm->start + segm->len) {
-		/* Remaining blocks to write */
-		blk_count = m2b(segm->len) - (blk - start_blk);
-		blk_count = MIN(blk_count, eckd_blk_max);
-		zero_page = get_zeroed_page();
-		writeblock(blk, addr, blk_count, zero_page);
-		free_page(zero_page);
-		progress_print(addr);
-		blk += blk_count;
-		addr += b2m(blk_count);
-	}
-	return blk;
+	return write_addr_range(blk, segm->start, segm->len, PRINT_PROGRESS);
 }
 
 /*
