@@ -15,9 +15,12 @@
 #include <stdint.h>
 #include <sys/types.h>
 
+#include "misc.h"
 #include "zipl.h"
 #include "lib/vtoc.h"
 
+#define FS_MAP_ERROR    "Could not get file mapping"
+
 
 /* Type for representing disk block numbers */
 typedef uint64_t blocknum_t;
@@ -101,6 +104,7 @@ int disk_type_is_eckd_ldl(struct disk_ext_type *ext_type);
 int disk_type_is_nvme(struct disk_ext_type *ext_type);
 int disk_type_is_eckd(disk_type_t type);
 
+int disk_info_set_fs_block(const char *filename, struct disk_info *info);
 int disk_get_info_from_file(const char* filename,
 			    struct job_target_data* target,
 			    struct disk_info** info);
@@ -113,13 +117,13 @@ int disk_head_from_blocknum(blocknum_t blocknum, struct disk_info* info);
 int disk_sec_from_blocknum(blocknum_t blocknum, struct disk_info* info);
 void disk_blockptr_from_blocknum(disk_blockptr_t* ptr, blocknum_t blocknum,
 				 struct disk_info* info);
-int disk_write_block_aligned(int fd, const void* data, size_t bytecount,
-	  		     disk_blockptr_t* block, struct disk_info* info);
-blocknum_t disk_write_block_buffer(int fd, int fd_is_basedisk,
+int disk_write_block_aligned(struct misc_fd *mfd, const void *data, size_t bytecount,
+			     disk_blockptr_t *block, struct disk_info *info);
+blocknum_t disk_write_block_buffer(struct misc_fd *fd, int fd_is_basedisk,
 				   const void* buffer, size_t bytecount,
 				   disk_blockptr_t** blocklist,
 				   struct disk_info *info);
-blocknum_t disk_write_block_buffer_align(int fd, int fd_is_basedisk,
+blocknum_t disk_write_block_buffer_align(struct misc_fd *mfd, int fd_is_basedisk,
 					 const void *buffer, size_t bytecount,
 					 disk_blockptr_t **blocklist,
 					 struct disk_info *info, int align,
