@@ -509,10 +509,7 @@ static void set_driver_name(int fd, struct disk_info *info, dev_t device)
 			info->drv_name = misc_strdup(dev_entry.name);
 		util_proc_dev_free_entry(&dev_entry);
 	} else {
-		fprintf(stderr, "Warning: Could not determine driver name for "
-			"major %d from /proc/devices\n", major(device));
-		fprintf(stderr, "Warning: Preparing a logical device for boot "
-			"might fail\n");
+		misc_warn_on_failed_pdge(device);
 	}
 }
 
@@ -811,20 +808,15 @@ int disk_type_is_eckd(disk_type_t type)
 		type == disk_type_eckd_cdl);
 }
 
-int
-disk_get_info_from_file(const char* filename, struct job_target_data* target,
-			struct disk_info** info)
+/**
+ * Retrieve and set block size of the file system which contains FILENAME
+ */
+int disk_info_set_fs_block(const char *filename, struct disk_info *info)
 {
-	struct stat stats;
-	char* device;
 	int blocksize;
 	int fd;
 	int rc;
 
-	if (stat(filename, &stats)) {
-		error_reason(strerror(errno));
-		return -1;
-	}
 	/* Retrieve file system block size */
 	fd = open(filename, O_RDONLY);
 	if (fd == -1) {
@@ -838,17 +830,33 @@ disk_get_info_from_file(const char* filename, struct job_target_data* target,
 			     filename);
 		return -1;
 	}
-	/* Create temporary device file */
-	rc = misc_temp_dev(stats.st_dev, 1, &device);
-	if (rc)
+	info->fs_block_size = blocksize;
+	return 0;
+}
+
+/**
+ * Retrieve disk info of the device which contains FILENAME
+ * and set the filesystem block size
+ */
+int disk_get_info_from_file(const char *filename,
+			    struct job_target_data *target,
+			    struct disk_info **info)
+{
+	struct stat stats;
+	char *device;
+
+	if (stat(filename, &stats)) {
+		error_reason(strerror(errno));
 		return -1;
-	/* Get device info */
-	rc = disk_get_info(device, target, info);
-	if (rc == 0)
-		(*info)->fs_block_size = blocksize;
-	/* Clean up */
+	}
+	if (misc_temp_dev(stats.st_dev, 1, &device))
+		return -1;
+	if (disk_get_info(device, target, info)) {
+		misc_free_temp_dev(device);
+		return -1;
+	}
 	misc_free_temp_dev(device);
-	return rc;
+	return disk_info_set_fs_block(filename, *info);
 }
 
 void disk_free_info(struct disk_info *info)
@@ -978,8 +986,8 @@ disk_blockptr_from_blocknum(disk_blockptr_t* ptr, blocknum_t blocknum,
  * otherwise. On success OFFSET contains offset of the first written byte
  */
 static int
-disk_write_block_aligned_base(int fd, int is_base_disk, const void* data,
-			      size_t bytecount, disk_blockptr_t* block,
+disk_write_block_aligned_base(struct misc_fd *mfd, int is_base_disk, const void *data,
+			      size_t bytecount, disk_blockptr_t *block,
 			      struct disk_info *info, int align, off_t *offset)
 {
 	blocknum_t current_block;
@@ -989,7 +997,7 @@ disk_write_block_aligned_base(int fd, int is_base_disk, const void* data,
 	if (align == 0)
 		align = info->phy_block_size;
 
-	current_pos = lseek(fd, 0, SEEK_CUR);
+	current_pos = lseek(mfd->fd, 0, SEEK_CUR);
 	if (current_pos == -1) {
 		error_text(strerror(errno));
 		return -1;
@@ -997,7 +1005,7 @@ disk_write_block_aligned_base(int fd, int is_base_disk, const void* data,
 	/* Ensure block alignment of current file pos */
 
 	if (current_pos % align != 0) {
-		current_pos = lseek(fd, align - current_pos % align, SEEK_CUR);
+		current_pos = lseek(mfd->fd, align - current_pos % align, SEEK_CUR);
 		if (current_pos == -1) {
 	       		error_text(strerror(errno));
 			return -1;
@@ -1008,11 +1016,11 @@ disk_write_block_aligned_base(int fd, int is_base_disk, const void* data,
 	if (bytecount > (size_t)info->phy_block_size)
 		bytecount = info->phy_block_size;
 	/* Write data block */
-	if (misc_write(fd, data, bytecount))
+	if (misc_write_or_simulate(mfd, data, bytecount))
 		return -1;
 	if (block != NULL) {
 		/* Store block pointer */
-		if (disk_get_blocknum(fd, is_base_disk, current_block,
+		if (disk_get_blocknum(mfd->fd, is_base_disk, current_block,
 				      &blocknum, info))
 			return -1;
 		disk_blockptr_from_blocknum(block, blocknum, info);
@@ -1022,10 +1030,10 @@ disk_write_block_aligned_base(int fd, int is_base_disk, const void* data,
 	return 0;
 }
 
-int disk_write_block_aligned(int fd, const void *data, size_t bytecount,
+int disk_write_block_aligned(struct misc_fd *mfd, const void *data, size_t bytecount,
 			     disk_blockptr_t *block, struct disk_info *info)
 {
-	return disk_write_block_aligned_base(fd, 0, data, bytecount, block,
+	return disk_write_block_aligned_base(mfd, 0, data, bytecount, block,
 					     info, info->phy_block_size, NULL);
 }
 
@@ -1039,7 +1047,7 @@ int disk_write_block_aligned(int fd, const void *data, size_t bytecount,
  * otherwise.
  */
 blocknum_t
-disk_write_block_buffer_align(int fd, int fd_is_basedisk, const void *buffer,
+disk_write_block_buffer_align(struct misc_fd *mfd, int fd_is_basedisk, const void *buffer,
 			      size_t bytecount, disk_blockptr_t **blocklist,
 			      struct disk_info *info, int align, off_t *offset)
 {
@@ -1054,7 +1062,7 @@ disk_write_block_buffer_align(int fd, int fd_is_basedisk, const void *buffer,
 	*blocklist = (disk_blockptr_t *)util_zalloc(sizeof(disk_blockptr_t) *
 						    count);
 	if (*blocklist == NULL) {
-		close(fd);
+		close(mfd->fd);
 		return 0;
 	}
 	/* Build list */
@@ -1062,7 +1070,7 @@ disk_write_block_buffer_align(int fd, int fd_is_basedisk, const void *buffer,
 		chunk_size = bytecount - written;
 		if (chunk_size > (size_t) info->phy_block_size)
 			chunk_size = info->phy_block_size;
-		rc = disk_write_block_aligned_base(fd, fd_is_basedisk,
+		rc = disk_write_block_aligned_base(mfd, fd_is_basedisk,
 					VOID_ADD(buffer, written),
 					chunk_size, &(*blocklist)[i],
 					info,
@@ -1077,11 +1085,11 @@ disk_write_block_buffer_align(int fd, int fd_is_basedisk, const void *buffer,
 }
 
 blocknum_t
-disk_write_block_buffer(int fd, int fd_is_basedisk, const void *buffer,
+disk_write_block_buffer(struct misc_fd *mfd, int fd_is_basedisk, const void *buffer,
 			size_t bytecount, disk_blockptr_t **blocklist,
 			struct disk_info *info)
 {
-	return disk_write_block_buffer_align(fd, fd_is_basedisk, buffer,
+	return disk_write_block_buffer_align(mfd, fd_is_basedisk, buffer,
 					     bytecount, blocklist, info,
 					     info->phy_block_size, NULL);
 }
