@@ -389,6 +389,8 @@ free_common_ipl_data(struct job_common_ipl_data *common)
 {
 	free(common->image);
 	free(common->parmline);
+	if (common->ramdisk_created)
+		unlink(common->ramdisk);
 	free(common->ramdisk);
 }
 
@@ -457,17 +459,24 @@ free_mvdump_data(struct job_mvdump_data* data)
 		free(data->device[i]);
 }
 
-
-void
-job_free(struct job_data* job)
+void job_free(struct job_data *job)
 {
-
-	if (job->dump_mounted && umount(job->target.bootmap_dir))
-		fprintf(stderr, "Could not umount dump device at %s",
-			job->target.bootmap_dir);
-	if (job->bootmap_dir_created && rmdir(job->target.bootmap_dir))
-		fprintf(stderr, "Could not remove directory %s",
-			job->target.bootmap_dir);
+	if (job->dump_mounted && umount(job->target.bootmap_dir)) {
+		fprintf(stderr,
+			"Could not umount dump device at %s(%s)\n",
+			job->target.bootmap_dir,
+			strerror(errno));
+	}
+	if (job->bootmap_dir_created) {
+		/* remove the meta-file created in dry-run mode */
+		ngdump_delete_meta(job->target.bootmap_dir);
+		if (rmdir(job->target.bootmap_dir)) {
+			fprintf(stderr,
+				"Could not remove directory %s(%s)\n",
+				job->target.bootmap_dir,
+				strerror(errno));
+		}
+	}
 	free_target_data(&job->target);
 	free_envblk_data(&job->envblk);
 	free(job->name);
@@ -967,6 +976,7 @@ check_job_images_ngdump(struct job_dump_data* dump, char *name)
 			value = NULL;
 		} else if (strcmp(param, "initrd") == 0) {
 			dump->common.ramdisk = value;
+			dump->common.ramdisk_created = true;
 			value = NULL;
 		} else if (strcmp(param, "cmdline") == 0) {
 			dump->common.parmline = value;
@@ -1956,7 +1966,7 @@ get_job_from_config_file(struct command_line* cmdline, struct job_data* job)
 static int get_job_envblk_data(struct job_data *job, char *import_hint)
 {
 	struct job_envblk_data *data = &job->envblk;
-	int fd;
+	struct misc_fd mfd = {0};
 
 	switch (job->id) {
 	case job_ipl:
@@ -1965,18 +1975,18 @@ static int get_job_envblk_data(struct job_data *job, char *import_hint)
 	default:
 		return 0;
 	}
-	fd = open(job->target.bootmap_dir, O_RDONLY);
-	if (fd < 0) {
+	mfd.fd = open(job->target.bootmap_dir, O_RDONLY);
+	if (mfd.fd < 0) {
 		error_reason(strerror(errno));
 		error_text("Could not open bootmap dir");
 		return -1;
 	}
-	if (envblk_size_get(fd, &data->size)) {
-		close(fd);
+	if (envblk_size_get(&mfd, &data->size)) {
+		close(mfd.fd);
 		error_text("Could not get environment block size");
 		return -1;
 	}
-	close(fd);
+	close(mfd.fd);
 	data->buf = misc_malloc(data->size);
 	if (data->buf == NULL) {
 		error_text("Could not allocate environment block");
