@@ -18,6 +18,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/sysmacros.h>
 #include <unistd.h>
 
 #include "error.h"
@@ -88,8 +89,7 @@ misc_strdup(const char* s)
 
 
 /* Open file exclusive */
-int
-misc_open_exclusive(const char* filename)
+static int misc_open_exclusive(const char *filename)
 {
 	int fd;
 
@@ -101,6 +101,35 @@ misc_open_exclusive(const char* filename)
 	return fd;
 }
 
+/**
+ * Open file FILENAME for write simulation
+ */
+static int misc_open_simulate(const char *filename, struct misc_fd *mfd)
+{
+	/*
+	 * In write simulation mode actual data are not written to the file,
+	 * so open it as read-only
+	 */
+	mfd->fd = open(filename, O_RDONLY);
+	mfd->simulate_write = 1;
+	if (mfd->fd == -1)
+		error_reason(strerror(errno));
+	return mfd->fd;
+}
+
+int misc_open_device(const char *filename, struct misc_fd *mfd, int simulate)
+{
+	if (simulate) {
+		/*
+		 * open the file for "write simulation", when actual
+		 * data are not written, and only current position in
+		 * the file is updated
+		 */
+		return misc_open_simulate(filename, mfd);
+	}
+	mfd->fd = misc_open_exclusive(filename);
+	return mfd->fd;
+}
 
 /* Read COUNT bytes of data from file identified by file descriptor FD to
  * memory at location BUFFER. Return 0 when all bytes were successfully read,
@@ -272,7 +301,7 @@ misc_get_char(struct misc_file_buffer* file, off_t readahead)
 
 
 char*
-misc_make_path(char* dirname, char* filename)
+misc_make_path(const char *dirname, char *filename)
 {
 	char* result;
 	size_t len;
@@ -404,6 +433,26 @@ misc_write(int fd, const void* data, size_t count)
 	return 0;
 }
 
+/**
+ * Instead of writing, upadte only current position in the file
+ */
+static int misc_simulate_write(int fd, size_t count)
+{
+	if (lseek(fd, count, SEEK_CUR) == (off_t)-1) {
+		error_reason(strerror(errno));
+		error_text("Could not update position in the file");
+		return -1;
+	}
+	return 0;
+}
+
+int misc_write_or_simulate(struct misc_fd *mfd, const void *data, size_t count)
+{
+	return mfd->simulate_write ?
+		misc_simulate_write(mfd->fd, count) :
+		misc_write(mfd->fd, data, count);
+}
+
 int misc_seek(int fd, off_t off)
 {
 	if (lseek(fd, off, SEEK_SET) == off)
@@ -614,4 +663,14 @@ void misc_ascii_to_ebcdic(unsigned char *from, unsigned char *to)
 		*from = ascebc[*from];
 }
 
-
+/**
+ *  zipl-specific response on failed util_proc_dev_get_entry()
+ */
+void misc_warn_on_failed_pdge(dev_t device)
+{
+	fprintf(stderr,
+		"Warning: Could not determine driver name for major %d from /proc/devices\n",
+		major(device));
+	fprintf(stderr,
+		"Warning: Preparing a logical device for boot might fail\n");
+}
