@@ -3,7 +3,7 @@
  *
  * Local helper functions
  *
- * Copyright 2017 IBM Corp.
+ * Copyright IBM Corp. 2017, 2020
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -25,10 +25,19 @@ static inline void misc_print_missing_command(void)
 	util_prg_print_parse_error();
 }
 
+/**
+ * Subcommand is missing
+ */
+static inline void misc_print_missing_sub_command(void)
+{
+	warnx("Subcommand is required");
+	util_prg_print_parse_error();
+}
+
 /**
  * Invalid command specified (for 'git' like tools)
  */
-void misc_print_invalid_command(const char *command)
+static void misc_print_invalid_command(const char *command)
 {
 	warnx("Invalid command '%s'", command);
 	util_prg_print_parse_error();
@@ -37,12 +46,25 @@ void misc_print_invalid_command(const char *command)
 /**
  * An required parameter has not been specified
  *
- * @param[in] option  Parameter string
+ * @param[in] parm_name  Parameter string
  */
-void misc_print_required_parm(const char *parm_name)
+static void misc_print_required_parm(const char *parm_name)
 {
 	warnx("Parameter '%s' is required", parm_name);
 	util_prg_print_parse_error();
 }
 
-#endif
\ No newline at end of file
+/**
+ * An required parameter has not been specified
+ *
+ * @param[in] parm_name1  Parameter string 1
+ * @param[in] parm_name2  Parameter string 2
+ */
+static void misc_print_required_parms(const char *parm_name1,
+				      const char *parm_name2)
+{
+	warnx("Parameter '%s' or '%s' is required", parm_name1, parm_name2);
+	util_prg_print_parse_error();
+}
+
+#endif
